--[[
Error handler by 0x25a0:
https://love2d.org/forums/viewtopic.php?f=5&t=83881
]]

local love = love

local format, gsub = string.format, string.gsub
local traceback = debug.traceback
local concat = table.concat

function love.errhand(error_message)
	local app_name = _gameTitle
	local version = "??"--_gameVersion
	local github_url = "https://www.github.com/DPlayer234/dpparkour2" -- no trailing slash
	--local email = "your-name@provider.ltd"
	local edition = love.system.getOS()

	local dialog_message = [[
%s crashed with the following error message:

%s

Would you like to report this crash so that it can be fixed?]]
	local titles = {"Oh no", "Oh boy", "Bad news"}
	local title = titles[love.math.random(#titles)]
	local full_error = traceback(error_message or "")
	local message = format(dialog_message, app_name, full_error)
	local buttons = {"Yes, on GitHub", "No", _arg.debug and "Debug" or nil} -- , "Yes, by email"

	local pressedbutton = love.window.showMessageBox(title, message, buttons)

	local function url_encode(text)
		-- This is not complete. Depending on your issue text, you might need to expand it!
		text = gsub(text, "\n", "%%0A")
		text = gsub(text, " ", "%%20")
		text = gsub(text, "#", "%%23")
		return text
	end

	local issuebody = [[
%s crashed with the following error message:

%s

[If you can, describe what you've been doing when the error occurred]

---
Version: %s
Edition: %s]]

	if pressedbutton == 1 then
		-- Surround traceback in ``` to get a Markdown code block
		full_error = concat({"```",full_error,"```"}, "\n")
		issuebody = format(issuebody, app_name, full_error, version, edition)
		issuebody = url_encode(issuebody)

		local subject = format("Crash in %s %s", app_name, version)
		local url = format("%s/issues/new?title=%s&body=%s", github_url, subject, issuebody)
		love.system.openURL(url)
	--[[elseif pressedbutton == 2 then
		issuebody = format(issuebody, app_name, full_error, version, edition)
		issuebody = url_encode(issuebody)

		local subject = format("Crash in %s %s", app_name, version)
		local url = format("mailto:%s?subject=%s&body=%s", email, subject, issuebody)
		love.system.openURL(url)]]
	elseif pressedbutton == 3 then
		if debug then debug.debug() end
	end
end
