local love = love
local pairs = pairs
local input, window, colors = input, window, colors

local display = {
	active = false
}

local active = false

local image, batch, data
local ids, state

function display.enable()
	local function newQuad(x, y, w, h)
		return love.graphics.newQuad(x, y, w, h, 32, 32)
	end

	-- Defining relevant data for usage
	data = {
		left = {
			x = 1, y = 4,
			quad = newQuad(1, 4, 3, 3)
		},
		right = {
			x = 7, y = 4,
			quad = newQuad(7, 4, 3, 3)
		},
		up = {
			x = 4, y = 1,
			quad = newQuad(4, 1, 3, 3)
		},
		down = {
			x = 4, y = 7,
			quad = newQuad(4, 7, 3, 3)
		},
		jump = {
			x = 14, y = 5,
			quad = newQuad(11, 5, 5, 5)
		},
		attack = {
			x = 20, y = 1,
			quad = newQuad(17, 1, 4, 5)
		},
	}

	-- Loading batch and quads
	image = assets.image("assets/input_display.png")
	batch = love.graphics.newSpriteBatch(image, 6, "dynamic")
	batch:setColor(colors.white(1, 1, 1, 0.5))

	ids = {}
	for k,v in pairs(data) do
		ids[k] = batch:add(v.quad, v.x, v.y)
	end

	state = {}
	for k,v in pairs(data) do
		state[k] = false
	end

	active = true
	display.active = true
end

function display.disable()
	image, batch, data,
	ids, state = nil

	active = false
	display.active = false
end

function display.draw(x, y, s)
	if active then
		for k,v in pairs(ids) do
			-- Updating the display
			local isDown = input.down(k)
			if state[k] ~= isDown then
				state[k] = isDown

				if isDown then
					batch:setColor(colors.white)
				else
					batch:setColor(colors.white(1, 1, 1, 0.5))
				end

				batch:set(v, data[k].quad, data[k].x, data[k].y)
			end
		end

		-- Drawing
		love.graphics.setColor(colors.black)
		love.graphics.draw(batch, x+s, y+s, 0, s)
		love.graphics.setColor(colors.white)
		love.graphics.draw(batch, x, y, 0, s)

		-- FPS
		love.graphics.printf(tostring(love.timer.getFPS()), x, 0, 78, "center", 0, s*(1/3))
	end
end

input.bindKey("f1", "inputDisplay")
input.bindEvent(function()
	if active then
		display.disable()
		game.message.queue("Disabled Input Display...", 0.3)
	else
		display.enable()
		game.message.queue("Enabled Input Display!", 0.3)
	end
end, "inputDisplay")

return display
