local love = love
local round = math.round
local insert, remove = table.insert, table.remove
local type, pairs = type, pairs
local window, colors = window, colors
local font = font

local log = newLog "[Message]"

local message = {}

local msg = {}
local queue = {}

local function toMsg(new)
	insert(msg, 1, new)

	local prev = new
	for i=2, #msg do
		local this = msg[i]
		if this.toY + this.h > prev.toY then
			this.toY = prev.toY - this.h
		end
		prev = this
	end
end

function message.queue(text, max)
	local w, lines = font:getWrap(text, 210)

	if not max then
		local length
		if type(text) == "table" then
			length = 0
			for k,v in pairs(text) do
				if type(v) == "string" then
					length = length + #v
				end
			end
		else
			length = #text
		end

		max = length * 0.075 + 1
	end

	local h = #lines*10 + 2
	local new = {
		text = text, age = 0, mage = max,
		y = -20, toY = -35 - h,
		w = w, h = h, fade = 0
	}

	if #msg < 7 then
		toMsg(new)
	else
		queue[#queue+1] = new
	end

	log("New:", text)
end

function message.clear()
	msg = {}
	queue = {}

	log("Cleared all messages...")
end

function message.update(dt)
	if #queue > 0 and #msg < 7 then
		toMsg(remove(queue, 1))
	end

	for i=#msg, 1, -1 do
		local v = msg[i]

		v.age = v.age + dt
		v.y = v.toY - (v.toY - v.y) * 0.075 ^ dt
		if v.age > v.mage then
			v.fade = v.fade * 0.01 ^ dt
			if v.fade < 0.01 then
				remove(msg, i)
			end
		else
			v.fade = 1 - (1 - v.fade) * 0.01 ^ dt
		end
	end
end

function message.draw()
	if #msg > 0 then
		love.graphics.push()

		love.graphics.scale(window.s)

		local centerX = round(window.vw * 0.5)
		local centerY = round(window.vh * 0.5)
		for i=1, #msg do
			local v = msg[i]

			love.graphics.setColor(colors.black(1, 1, 1, v.fade*0.5))
			love.graphics.rectangle("fill", centerX - v.w*0.5 - 1, centerY + v.y, v.w + 2, v.h - 2)
			love.graphics.setColor(colors.black(1, 1, 1, v.fade))
			love.graphics.printf(v.text, centerX - 104, centerY + v.y + 1, 210, "center")
			love.graphics.setColor(colors.white(1, 1, 1, v.fade))
			love.graphics.printf(v.text, centerX - 105, centerY + v.y, 210, "center")
		end

		love.graphics.setColor(colors.white)

		love.graphics.pop()
	end
end

return message
