local love = love
local window = window

local post = {}

local canvas = love.graphics.newCanvas(1, 1)
local width, height = 1, 1

local shader, update

function post.setup(path)
	local data = require(path)
	shader = data.shader
	update = data.update
end

function post.start()
	if width ~= window.w or height ~= window.h then
		canvas = love.graphics.newCanvas(window.w, window.h)
		width, height = window.w, window.h
	end

	love.graphics.setCanvas(canvas)
	update(shader, canvas)
end

function post.fin()
	love.graphics.setCanvas(nil)
	love.graphics.setShader(shader)

	love.graphics.push()
	love.graphics.origin()
	love.graphics.draw(canvas)
	love.graphics.pop()

	love.graphics.setShader(nil)
end

function update(shader, canvas)
	shader:send("WIDTH", window.w)
	shader:send("HEIGHT", window.h)
end

shader = love.graphics.newShader[[
	uniform float WIDTH = 1;
	uniform float HEIGHT = 1;

	vec4 effect(vec4 vcolor, Image texture, vec2 coord, vec2 pixel_coords) {
		float x = 1/WIDTH;
		float y = 1/HEIGHT;

		vec4 x0y0 = Texel(texture, coord);
		vec4 x1y0 = Texel(texture, coord+vec2(x, 0));
		vec4 x2y0 = Texel(texture, coord+vec2(-x, 0));
		vec4 x0y1 = Texel(texture, coord+vec2(0, y));
		vec4 x1y1 = Texel(texture, coord+vec2(x, y));
		vec4 x2y1 = Texel(texture, coord+vec2(-x, y));
		vec4 x0y2 = Texel(texture, coord+vec2(0, -y));
		vec4 x1y2 = Texel(texture, coord+vec2(x, -y));
		vec4 x2y2 = Texel(texture, coord+vec2(-x, -y));

		return (x0y0 + x1y0 + x2y0 + x0y1 + x1y1 + x2y1 + x0y2 + x1y2 + x2y2)/9;
	}
]]

return post
