local love = love
local sin, roundn = math.sin, math.roundn
local colors = colors

local Barrier = {}

Barrier.quads = {
	hl = love.graphics.newQuad( 0,  0, 9, 9, 39, 20),
	hc = love.graphics.newQuad(10,  0, 9, 9, 39, 20),
	hr = love.graphics.newQuad(20,  0, 9, 9, 39, 20),
	hs = love.graphics.newQuad(30,  0, 9, 9, 39, 20),
	vt = love.graphics.newQuad( 0, 11, 9, 9, 39, 20),
	vc = love.graphics.newQuad(10, 11, 9, 9, 39, 20),
	vb = love.graphics.newQuad(20, 11, 9, 9, 39, 20),
	vs = love.graphics.newQuad(30, 11, 9, 9, 39, 20)
}

function Barrier:new(x, y, z, hori, len, req)
	self:__new(x, y, 1, 1, 1)

	self.time = 0

	self.mass = 1e300
	self.floating = true

	self.close = 0
	self.text = req and {
		colors.white, "Clear ",
		colors.pRed, req,
		colors.white, "."
	} or false

	self.image = assets.image("assets/object/barrier.png")
	self.batch = love.graphics.newSpriteBatch(self.image, len, "static")

	self.__x = x
	self.__y = y
	if hori then
		self.__w = len
		self.__h = 1

		if len == 1 then
			self.batch:add(self.quads.hs, 0, 0)
		else
			self.batch:add(self.quads.hl, 0, 0)
			for i=1, len-2 do
				self.batch:add(self.quads.hc, i*9, 0)
			end
			self.batch:add(self.quads.hr, len*9-9, 0)
		end
	else
		self.__w = 1
		self.__h = len

		if len == 1 then
			self.batch:add(self.quads.vs, 0, 0)
		else
			self.batch:add(self.quads.vt, 0, 0)
			for i=1, len-2 do
				self.batch:add(self.quads.vc, 0, i*9)
			end
			self.batch:add(self.quads.vb, 0, len*9-9)
		end
	end

	self.data.ground = true
end

function Barrier:init()
	local ts = self.tiles.ts
	local w = self.__w
	local h = self.__h

	self.x = self.__x*ts-ts
	self.y = self.__y*ts-ts
	self.w = w*ts
	self.h = h*ts

	self.s = ts/9
end

function Barrier:hasColided(other, dir, dt)
	if dir == "x+" or dir == "x-" then
		self.xv = 0
		other.xv = 0
	elseif dir == "y+" or dir == "y-" then
		self.yv = 0
		other.yv = 0
		self.data.onTop = false
	end
end

function Barrier:hurt()
	self.xv = 0
	self.yv = 0
end

function Barrier:kill()
	self.tiles.particle:spawn("smoke", self.w*self.h*0.2, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:remove()
end

function Barrier:update(dt)
	self.time = self.time + dt

	if self.text then
		local distance = (((self.x+self.w*0.5)-(jnr.player.x+jnr.player.w*0.5))^2 + ((self.y+self.h*0.5)-(jnr.player.y+jnr.player.h*0.5))^2)^0.5
		if distance < 31 then
			if self.close < 1 then
				self.close = self.close + dt * 3
				if self.close > 1 then self.close = 1 end
			end
		elseif self.close > 0 then
			self.close = self.close - dt * 3
			if self.close < 0 then self.close = 0 end
		end
	end
end

function Barrier:draw(s)
	local c = sin(self.time)*0.25+0.75
	love.graphics.setColor(colors.white(1, c, c, 1))

	love.graphics.draw(self.batch, self.x, self.y, 0, self.s)

	love.graphics.setColor(colors.white)
end

function Barrier:drawL(s)
	if self.close > 0 then
		local x = roundn(self.x, s)
		local y = roundn(self.y, s)

		love.graphics.setColor(colors.black(1, 1, 1, self.close))
		love.graphics.print(self.text, x + self.w + 3, y)

		love.graphics.setColor(colors.white(1, 1, 1, self.close))
		love.graphics.print(self.text, x + self.w + 2, y - 1)

		love.graphics.setColor(colors.white)
	end
end

return class("Barrier", require("jnr.object"), Barrier)
