local love = love
local abs, roundn, sin, asin, pi = math.abs, math.roundn, math.sin, math.asin, math.pi
local colors = colors

local PurpleSister = {}

function PurpleSister:new()
	self.floaty = 1.5 * pi
	self.__yrOff = self.yrOff

	self.pJumps = 1
	self.countGJump = true

	self.floating = true
	self.float = {
		active = false,
		time = 0,
		maxTime = 0.2,
		hold = 0,
		dustTime = 0,
		barAlpha = 0,
		barFillLast = 1,
		barBase = love.graphics.newQuad(60, 20, 24, 4, self.anim.img:getDimensions()),
		barFore = love.graphics.newQuad(60, 25, 0, 4, self.anim.img:getDimensions())
	}

	self.speed.wall = self.speed.normal

	self.jump.normal = -190
	self.jump.wall = -150
	self.jump.air = -150

	self.canSwim = false
	self.canWallRun = false
	self.canWallJump = true
	self.mass = 5 * 17

	self.draw = PurpleSister.draw
	self.drawL = PurpleSister.drawL

	self.wings = self.anim:getQuad(5, 1)
end

function PurpleSister:always(dt)
	local float = self.float

	self.floating = not self.data.duck

	if self.data.ground or self.data.wall then
		float.active = false
	end

	-- "Flying" mechanics
	if self.ctrl.up then
		float.active = true
		self.jumped = false
		if float.time < float.maxTime then
			-- Still able to rise
			self.yvLimit = -190
			self.yvAcc   = 0.02

			-- Expending Energy
			if self.data.water then
				float.time = float.time + dt * 0.1
			elseif self.data.wall then
				float.time = float.time + dt * 1.5
			else
				float.time = float.time + dt
			end
			if float.time > float.maxTime then
				float.time = float.maxTime
			end
		elseif not self.data.wall then
			-- Only floating
			self.yvLimit = 256
			self.yvAcc   = 0.4
		end

		-- Effects
		if float.hold < 1 then
			float.hold = float.hold + dt * 4
			if float.hold > 1 then float.hold = 1 end
		end
	else
		-- Effects
		if float.hold > 0 then
			float.hold = float.hold - dt * 2
			if float.hold < 0 then float.hold = 0 end
		end

		if float.time > 0 then
			-- Recovering Energy
			if self.data.ground then
				float.time = float.time - dt * 2
			elseif self.data.wall then
				float.time = float.time - dt
			else
				float.time = float.time - dt * 0.35
			end
			if float.time < 0 then float.time = 0 end
		end
	end

	if float.hold > 0 then
		float.dustTime = float.dustTime + float.hold*float.hold*dt*60
		if float.dustTime > 1 then
			if self.dir < 0 then
				self.tiles.particle:spawn("dust", float.dustTime, self.x + self.w + 2, self.y + self.h * 0.5, 2, self.h * 0.3, self.xv * -0.5, self.yv * -0.5, 1)
			else
				self.tiles.particle:spawn("dust", float.dustTime, self.x - 2, self.y + self.h * 0.5, 2, self.h * 0.3, self.xv * -0.5, self.yv * -0.5, -1)
			end
			float.dustTime = float.dustTime % 1
		end
	end

	-- Updating direction while flying
	if float.active then
		if self.ctrl.right ~= self.ctrl.left then
			self.dir = self.ctrl.right and 1 or -1
		end
	end

	if float.active or float.time > 0 then
		if float.barAlpha < 1 then
			float.barAlpha = float.barAlpha + dt * 3
			if float.barAlpha > 1 then float.barAlpha = 1 end
		end
	elseif float.barAlpha > 0 then
		float.barAlpha = float.barAlpha - dt * 2
		if float.barAlpha < 0 then float.barAlpha = 0 end
	end

	-- Bobbing
	if self.data.ground and not self.data.duck then
		self.floaty = self.floaty + dt * (3 + abs(self.xv * 0.08))
		self.yrOff = sin(self.floaty) + self.__yrOff + 1
	elseif self.yrOff > self.__yrOff then
		self.yrOff = self.__yrOff - (self.__yrOff - self.yrOff) * 0.001 ^ dt
		self.floaty = asin(self.yrOff - self.__yrOff - 1)
	end
end

function PurpleSister:slide(dt)
	self.xvAcc = 1.5e-7
end

function PurpleSister:draw(s)
	local hold = self.float.hold * self.float.hold
	love.graphics.setColor(colors.white(1-hold*0.16, 1-hold*0.5, 1, self.dead or 1))
	love.graphics.draw(self.anim.img, self.wings, roundn(self.x + self.xrOff, s), roundn(self.y - self.yrOff, s), 0, self.dir, (self.data.duck and 0.5 or 1), self.xrCen + 2, 0)
	love.graphics.setColor(colors.white)

	self.anim:draw(roundn(self.x + self.xrOff, s), roundn(self.y - (self.data.duck and self.h or 0) - self.yrOff, s), 0, self.dir, 1, self.xrCen, 0)

	if self.attackHitbox then
		local hitbox = self.attackHitbox
		self.attackAnim:draw(roundn(hitbox.x + (self.attackDir > 0 and 0 or hitbox.w), s), roundn(hitbox.y - self.yrOff + self.__yrOff, s), 0, self.attackDir, hitbox.h/17, 0, 1)
	end
end

function PurpleSister:drawL(s)
	local float = self.float

	if float.barAlpha > 0 then
		local fill = 1 - float.time / float.maxTime
		if float.barFillLast ~= fill then
			float.barFore:setViewport(60, 20, 24*fill, 4)
			float.barBase:setViewport(60+24*fill, 25, 24-24*fill, 4)

			float.barFillLast = fill
		end

		local left = roundn(self.x + self.w*0.5 - 12, s)
		local top = roundn(self.y + self.h + 2, s)
		love.graphics.setColor(colors.white(1, 1, 1, 0.75*float.barAlpha))
		love.graphics.draw(self.anim.img, float.barFore, left, top)
		love.graphics.draw(self.anim.img, float.barBase, left+24*fill, top)
		love.graphics.setColor(colors.white)
	end
end

return {
	code = PurpleSister,
	new = PurpleSister.new,
	anim = anim.newHeader(assets.image("assets/char/purple_sister.png"), 15, 19)
		:spc("idle" , {1,1, 2,1, 3,1, 2,1, 1,1, 4,1}, 5, true)
		:spc("walk" , {1,2, 2,2, 1,2, 4,2}, 6, true)
		:reg("jump" , 1, 3, 3, 14, false)
		:reg("wall" , 4, 3, 1, 1, false)
		:reg("duck" , 1, 4, 1, 1, false)
		:spc("crawl", {2,4, 1,4, 3,4, 1,4}, 12, true)
		:reg("slide", 4, 4, 1, 1, false)
		:spc("push" , {1,5, 2,5, 3,5, 2,5, 1,5, 4,5}, 5, true)
		:reg("attack", 5, 5, 3, 25, false)
		:reg("dead" , 1, 6, 8, 30, true)
		:copy("wall", "wallrun"),
	imageP2 = "assets/char/p2/purple_sister.png",
	name = "Purple Sister",
	desc = {
		colors.white, "Also known as Nepgear. She's the CPU Candidate of Planeptune, in a dimension far, far away.\nHold ",
		colors.yellow, input.getPrompt("up"),
		colors.white, " to fly/hover. Her ",
		colors.yellow, "jumps are shorter",
		colors.white, " and she ",
		colors.yellow, "can't do wall-runs or air-jumps at all",
		colors.white, ".\n",
		colors.softBlue, "Hyperdimension Neptunia and associated characters are © Idea Factory, Compile Heart."
	}
}
