local roundn = math.roundn
local sfx = sfx

local CheckPoint = {}

CheckPoint.anim = anim.newHeader(assets.image("assets/object/check_flag.png"), 11, 13)
	:reg(0, 1, 1, 8, 12, true)

function CheckPoint:new(x, y, z)
	self:__new(x, y, 3, 11, -2)
	self.canColide = false
	self.mass = 13

	self.anim = self.anim:clone():set(0)
end

function CheckPoint:hasColided(other, dir, dt)
	if other.type == "Player" then
		if not other.dead then
			self:kill()
			self.tiles:backup()
		end
	end
end

function CheckPoint:update(dt)
	-- Acceleration (Gravity/Air/Ground)
	if self.data.ground then
		self.xvAcc = 1e-14
	else
		self.xvAcc = 0.008
	end

	if self.data.wall then
		self.yvLimit = 30
		self.yvAcc   = 6e-7
	else
		self.yvLimit = 512
		self.yvAcc   = 0.4
	end

	self:accelerate(dt)
	self:colide(dt)

	self.anim:update(dt)
	if self.y > self.tiles.height * self.tiles.ts then
		self:kill()
	end

	self:spawnParticles(dt)
	self:makeSounds()
end

function CheckPoint:kill()
	self.tiles.particle:spawn("smoke", 10, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:remove()
end

function CheckPoint:hurt()
	local vol = (self.xv*self.xv+self.yv*self.yv)^0.5 * 0.0016
	sfx.colide:play(self.x + self.w*0.5, self.y + self.h*0.5, vol)
end

function CheckPoint:draw(s)
	self.anim:draw(roundn(self.x - 1, s), roundn(self.y - 1, s))
end

return class("CheckPoint", require("jnr.object"), CheckPoint)
