local abs, roundn = math.abs, math.roundn
local ipairs = ipairs
local sfx = sfx

local boss1 = {
	w = 34, h = 27, mass = 800,
	anim = anim.newHeader(assets.image("assets/entity/boss1.png"), 36, 29)
		:reg("idle", 1,1, 2, 6, true)
		:reg("dead", 3,1, 2, 12, true)
		:reg("rush!", 1,2, 4, 8, false)
		:spc("rush", {2,3, 1,3, 3,3, 1,3}, 32, true)
		:reg("leap!", 1,4, 4, 8, false)
		:reg("leap", 1,5, 3, 6, false)
		:reg("leap2", 4,5, 1, 1, false)
		:reg("stomp!", 1,6, 4, 8, false)
		:reg("stomp", 1,7, 4, 6, false),
	states = {}
}

local Entity = require "jnr.entity"
local shockWave = jnr.getEntityData "boss1_shock"

local player, hardcore

function boss1:new(isHardcore)
	self.bar = jnr.getBossBar()

	self.hp = 30
	self.dir = -1

	self.state = "idle"
	self.time = 0
	self.stomps = 0

	self.step = {
		delay = 0.0625,
		time = 0
	}

	self.random = assets.random("boss")

	self.anim = boss1.anim:clone()
	self.anim:set "idle"

	self.xOff = 0
	self.yOff = 0

	hardcore = isHardcore or false
end

local function chooseTarget(self)
	local players = {}
	for i,v in ipairs(self.tiles.object) do
		if v.type == "Player" then
			players[#players+1] = v
		end
	end

	local target = players[self.random:random(1, #players)]
	if target then player = target end
end

function boss1:init()
	for i,v in ipairs(self.tiles.object) do
		if v.type == "Player" then
			player = v
			return
		end
	end

	-- This code should never execute.
	game.message.queue "The Boss could not find you and decided to just leave.\nAka this is a bug you should report.\n:ERROR:boss1.init"
	self:remove()
end

--[[
All States
]]
function boss1.states:idle(dt)
	self.anim:set "idle"
	if self.data.ground then
		if self.time > (hardcore and 1 or 1.5) then
			self.time = 0

			if player.x + player.w * 0.5 < self.x + self.w * 0.5 then
				self.dir = -1
			else
				self.dir = 1
			end

			local next = self.random:random(1, 3)
			if next == 1 then
				self.state = "rush"
			elseif next == 2 then
				self.state = "leap"
			elseif next == 3 then
				self.stomps = 0
				self.state = "stomp"
			end
		end
	elseif self.time > 0 then
		self.time = self.time - dt * 1.5
	end
end

function boss1.states:rush(dt)
	if self.time < (hardcore and 0.5 or 1) then
		self.anim:set "rush!"

		if player.x + player.w * 0.5 < self.x + self.w * 0.5 then
			self.dir = -1
		else
			self.dir = 1
		end
	else
		self.anim:set "rush"

		if hardcore then
			self.xvLimit = self.dir * 480
		else
			self.xvLimit = self.dir * 320
		end

		self.step.time = self.step.time + dt
		if self.step.time > self.step.delay then
			if self.data.ground then sfx.colide:play(self.x + self.w*0.5, self.y + self.h, 0.25) end
			self.step.time = self.step.time % self.step.delay
		end

		if self.dir > 0 and self.x > player.x or self.dir < 0 and self.x + self.w < player.x + player.w then
			self.xvLimit = 0

			self.state = "idle"
			self.time = 0
			chooseTarget(self)
		end
	end
end

function boss1.states:leap(dt)
	if self.time < (hardcore and 0.5 or 1) then
		self.anim:set "leap!"

		if player.x + player.w * 0.5 < self.x + self.w * 0.5 then
			self.dir = -1
		else
			self.dir = 1
		end
	else
		self.anim:set "leap"
		if hardcore then
			self.yv = -240
			self.xv = ((player.x + player.w * 0.5) - (self.x + self.w * 0.5)) * 1.1
		else
			self.yv = -320
			self.xv = ((player.x + player.w * 0.5) - (self.x + self.w * 0.5)) * 0.85
		end
		self.xvLimit = self.xv

		sfx.colide:play(self.x + self.w*0.5, self.y + self.h, 0.35)

		self.state = "leapAir"
	end
end

function boss1.states:leapAir(dt)
	if self.yv < 0 then
		self.anim:set "leap"
	else
		self.anim:set "leap2"
	end

	if self.data.ground then
		self.xvLimit = 0

		self.state = "idle"
		self.time = 0
		chooseTarget(self)
	end
end

function boss1.states:stomp(dt)
	if self.time < (hardcore and 0.5 or 1) then
		self.anim:set "stomp!"

		if player.x + player.w * 0.5 < self.x + self.w * 0.5 then
			self.dir = -1
		else
			self.dir = 1
		end
	else
		self.anim:set "stomp"
		if self.data.ground then
			if hardcore then
				self.xv = self.random:random()*160-80
				self.xvLimit = self.xv
			end

			if self.stomps > 0 then
				local speed = hardcore and 1.5 or 1
				local tiles = self.tiles

				local spawn1 = Entity(self.x/tiles.ts, (self.y+self.h)/tiles.ts, 1, shockWave, -speed)
				local spawn2 = Entity((self.x+self.w)/tiles.ts+1, (self.y+self.h)/tiles.ts, 1, shockWave, speed)

				tiles:addObjects(spawn1, spawn2)
			end
			self.stomps = self.stomps + 1

			if self.stomps < 4 or (hardcore and self.stomps < 7) then
				self.anim:reset()
				self.yv = -120
			else
				self.xvLimit = 0
				self.time = 0
				self.state = "idle"
			end
		end
	end
end

function boss1.states:dead(dt)
	self.xOff = self.random:random(-2, 2)
	self.yOff = self.random:random(-2, 2)

	self.anim:set "dead"
	self.tiles.particle:spawn("smoke", dt*60, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)

	if self.time > 2.5 then
		local Entity = require "jnr.entity"
		local coinData = jnr.getEntityData "coin"
		for i=1, 9 do
			local coin = Entity(0, 0, -1, coinData)
			self.tiles:addObject(coin)
			coin.x = self.random:random(self.x, self.x + self.w - coin.w)
			coin.y = self.random:random(self.y, self.y + self.w - coin.h - 9)
		end

		local finish = require "jnr.finish" (self.tiles.width*0.5, -18, -1)
		self.tiles:addObject(finish)

		self:kill()
		self.tiles:backup()
		jnr.clearCondition = jnr.clearConditions.playerIsDead
	end
end

--[[
Entity Code
]]
function boss1:hurt()
	local anim = self.anim.anim
	if anim == "rush!" or anim == "leap!" or anim == "stomp!" then
		self.hp = self.hp - 1/2
	else
		self.hp = self.hp - 1
	end
	self.bar(self.hp/30)

	self.tiles.particle:spawn("smoke", 10, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)

	if self.hp <= 0 and self.state ~= "dead" then
		self.state = "dead"
		self.time = 0

		self.xvLimit = 0
		sfx.explode:play(self.x + self.w*0.5, self.y + self.h*0.5)

		if not save.data.flags[jnr.currentLevel] and not hardcore then
			game.setState("scene", "1-4_fin")
		end
	end

	jnr.shake(0.15)
	sfx.smalexp:play(self.x + self.w*0.5, self.y + self.h*0.5)
	sfx.colide:play(self.x + self.w*0.5, self.y + self.h*0.5, 1)
end

function boss1:kill(depth)
	if depth then
		self.x = self.tiles.width * self.tiles.ts * 0.5 - self.w * 0.5
		self.y = -256
	else
		self.tiles.particle:spawn("smoke", 210, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)

		jnr.scrollOverride()
		self:remove()
	end
end

function boss1:reset()
	self:__reset()
	self.bar(self.hp/30)
end

function boss1:hasColided(other, dir, dt)
	if self.hp > 0 then
		if other.type == "Player" then
			if (dir == "x+" and self.xv > 1) or (dir == "x-" and self.xv < -1) or dir == "y+" then
				other:hurt(self)

				if self.state == "rush" then
					other.xv = other.xv * 3.5
					self.xvLimit = 0

					self.state = "idle"
					self.time = 0
					chooseTarget(self)
				elseif self.state == "leapAir" and dir == "y+" then
					self.xvLimit = 0
					self.yv = -210
					self.data.ground = false

					if player.playerId == "denniz" and not unlocks.small_denniz then
						game.queueTask(save.unlock, "small_denniz", "char")
					end
				elseif self.state == "leapAir" and (dir == "x+" or dir == "x-") then
					self.xvLimit = 0
				end
			elseif dir == "n" and not other.dead then
				other:hurt(self)
			end
		end
	end
end

function boss1:preAcc(dt)
	if not player.dead then
		jnr.scrollOverride(
			(player.x + player.w*0.5 + self.x + self.w*0.5)*0.5,
			(player.y + player.h*0.5 + self.y + self.h*0.5)*0.5
		)
	end
end

function boss1:postAcc(dt)
	self.time = self.time + dt
	boss1.states[self.state](self, dt)
end

function boss1:postCol(dt)
	self.anim:update(dt)

	if self.data.ground and not self.pdata.ground then
		jnr.shake(abs(self.yv-self.pdata.yv) * 0.003)
	end
	if self.data.wall and not self.pdata.wall then
		jnr.shake(abs(self.xv-self.pdata.xv) * 0.003)
	end
end

function boss1:draw(s)
	if self.state == "dead" then
		self.anim:draw(roundn(self.x + 17, s) + self.xOff, roundn(self.y - 1, s) + self.yOff, 0, self.dir, 1, 17)
	else
		self.anim:draw(roundn(self.x + 17, s), roundn(self.y - 1, s), 0, self.dir, 1, 17)
	end
end

return boss1
