local roundn = math.roundn

local buggy = {
	w = 9, h = 9,
	anim = anim.newHeader(assets.image("assets/entity/buggy.png"), 13, 11)
		:spc("walk", {1,1, 2,1, 1,1, 3,1}, 20, true)
		:reg("dead", 1,2, 3, 3, false)
}

function buggy:new()
	self.dead = false
	self.dir = 1

	self.xvLimit = 50
	self.xvAcc   = 1.5e-7

	self.anim = buggy.anim:clone()
	self.anim:set "walk"
end

function buggy:hurt(who)
	self.dead = true
	self.anim:set("dead"):reset()

	if who.type == "Player" then
		self.yv = -70
		self.xv = self.xv * 0.25
		self.dir = self.xv > 0 and -1 or 1
	end
	self.xvLimit = 0
	self.xvAcc   = 0.01

	self.tiles.particle:spawn("blood", 10, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5, self.xv, self.yv)
end

function buggy:hasColided(other, dir, dt)
	if not self.dead and other.type == "Player" then
		if dir ~= "y+" then
			other:hurt(self)
			if dir == "y-" then
				other.yv = -70
			end
		end
	end
end

function buggy:kill()
	self.tiles.particle:spawn("smoke", 15, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:remove()
end

function buggy:reset()
	self:__reset()
	if self.dead then
		self.anim:set "dead"
	else
		self.anim:set "walk"
	end
end

function buggy:preAcc(dt) end

function buggy:postAcc(dt) end

function buggy:postCol(dt)
	if self.dead then
		if self.anim:update(dt) then
			self:kill()
		end
	else
		if self.data.wall then
			self.xv = -self.pdata.xv
			self.xvLimit = -self.xvLimit
		end
		self.dir = self.xvLimit > 0 and 1 or -1

		self.anim:update(dt)
	end
end

function buggy:draw(s)
	self.anim:draw(roundn(self.x + 4, s)+0.5, roundn(self.y - 1, s), 0, self.dir, 1, 6.5)
end

return buggy
