local abs, roundn = math.abs, math.roundn

local flyer = {
	w = 9, h = 9,
	anim = anim.newHeader(assets.image("assets/entity/flyer.png"), 13, 11)
		:spc("idle", {1,1, 2,1, 1,1, 3,1}, 20, true)
		:reg("dead", 1,2, 3, 3, false)
}

function flyer:new()
	self.dead = false
	self.dir = 1

	self.xvLimit = 0
	self.xvAcc   = 0.2
	self.flightdir = 1
	self.yvAcc   = 0.2

	self.anim = flyer.anim:clone()
	self.anim:set "idle"
end

function flyer:init()
	self.__x = self.x
	self.__y = self.y

	self.mass = 18000
	self.effects = false
end

function flyer:hurt(who)
	self.dead = true
	self.anim:set("dead"):reset()

	if who.type == "Player" then
		self.yv = -70
		if self.mass > 81 then
			self.xv = self.xv * 50
		else
			self.xv = self.xv * 0.25
		end
		self.dir = self.xv > 0 and -1 or 1
	end
	self.xvLimit = 0
	self.xvAcc   = 0.01

	self.mass = 81
	self.effects = true

	self.tiles.particle:spawn("blood", 10, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5, self.xv, self.yv)
end

function flyer:hasColided(other, dir, dt)
	if not self.dead then
		if dir == "x+" or dir == "x-" then
			if other.type == "Player" then
				other:hurt(self)
			end
			self.dir = dir == "x+" and 1 or -1
		else
			if self.yv < 0 and dir == "y-" then
				other.yv = 1
			end
			if other.dir then
				self.dir = other.dir
			end
		end
	end
end

function flyer:kill()
	self.tiles.particle:spawn("smoke", 15, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:remove()
end

function flyer:reset()
	self:__reset()
	self.anim:set "idle"
end

function flyer:preAcc(dt)
	if self.dead then
		self.xvLimit = 0
		self.yvLimit = 512
		self.yvAcc   = 0.4
	else
		if self.__x ~= self.x then
			self.xvLimit = (self.__x - self.x) * 2
		end
		self.yvLimit = 50 * self.flightdir
	end
end

function flyer:postAcc(dt) end

function flyer:postCol(dt)
	if self.dead then
		if self.anim:update(dt) then
			self:kill()
		end
	else
		if self.flightdir > 0 then
			if self.__y + 45 < self.y then
				self.flightdir = -self.flightdir
			end
		elseif self.flightdir < 0 then
			if self.__y - 45 > self.y then
				self.flightdir = -self.flightdir
			end
		end

		self.anim:update(dt)
	end
end

function flyer:draw(s)
	self.anim:draw(roundn(self.x + 4, s)+0.5, roundn(self.y - 1, s), 0, self.dir, 1, 6.5)
end

return flyer
