local roundn = math.roundn
local sfx = sfx

local Finish = {}

Finish.anim = anim.newHeader(assets.image("assets/object/finish_flag.png"), 11, 29)
	:reg(0, 1, 1, 8, 12, true)

function Finish:new(x, y, z)
	self:__new(x, y, 3, 27, -2)
	self.canColide = false
	self.mass = 29

	self.anim = self.anim:clone():set(0)
end

function Finish:hasColided(other, dir, dt)
	if other.type == "Player" then
		if not other.dead then
			jnr.clearCondition = true
		end
	end
end

function Finish:update(dt)
	-- Acceleration (Gravity/Air/Ground)
	if self.data.ground then
		self.xvAcc = 1e-14
	else
		self.xvAcc = 0.008
	end

	if self.data.wall then
		self.yvLimit = 30
		self.yvAcc   = 6e-7
	else
		self.yvLimit = 512
		self.yvAcc   = 0.4
	end

	self:accelerate(dt)
	self:colide(dt)

	self.anim:update(dt)
	if self.y > self.tiles.height * self.tiles.ts then
		self:respawn()
	end

	self:spawnParticles(dt)
	self:makeSounds()
end

function Finish:respawn()
	self.tiles.particle:spawn("smoke", 30, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:reset()
	self.tiles.particle:spawn("smoke", 30, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
end

function Finish:hurt()
	local vol = (self.xv*self.xv+self.yv*self.yv)^0.5 * 0.0016
	sfx.colide:play(self.x + self.w*0.5, self.y + self.h*0.5, vol)
end

function Finish:draw(s)
	self.anim:draw(roundn(self.x - 1, s), roundn(self.y - 1, s))
end

return class("Finish", require("jnr.object"), Finish)
