local love = love
local sin, pi, roundn = math.sin, math.pi, math.roundn

local Platform = {}

Platform.quads = {
	l = love.graphics.newQuad( 0, 0, 9, 9, 39, 9),
	c = love.graphics.newQuad(10, 0, 9, 9, 39, 9),
	r = love.graphics.newQuad(20, 0, 9, 9, 39, 9),
	s = love.graphics.newQuad(30, 0, 9, 9, 39, 9)
}

function Platform:new(x, y, z, len, dist, left, vert)
	self:__new(x, y, 1, 1, z)

	self.dist = dist
	self.move = not left
	self.vert = vert or false

	self.mass = 1e300
	self.floating = true

	self.image = assets.image("assets/object/platform.png")
	self.batch = love.graphics.newSpriteBatch(self.image, len, "static")

	self.xvLimit = 0
	self.xvAcc   = self.vert and 0 or 0.4
	self.yvLimit = 0
	self.yvAcc   = self.vert and 0.4 or 0

	self.__x = x
	self.__y = y

	self.__w = len
	self.__h = 1

	if len == 1 then
		self.batch:add(self.quads.s, 0, 0)
	else
		self.batch:add(self.quads.l, 0, 0)
		for i=1, len-2 do
			self.batch:add(self.quads.c, i*9, 0)
		end
		self.batch:add(self.quads.r, len*9-9, 0)
	end
end

function Platform:init()
	local ts = self.tiles.ts
	local w = self.__w
	local h = self.__h

	self.x = self.__x*ts-ts
	self.y = self.__y*ts-ts
	self.w = w*ts
	self.h = h*ts*(2/3)

	self.__x = self.x
	self.__y = self.y
	self.__w = nil
	self.__h = nil

	self.s = ts/9
	self.ts = ts
end

function Platform:hasColided(other, dir, dt)
	if self.vert and self.yv < 0 and dir == "y-" then
		other.yv = 1
	end
end

function Platform:update(dt)
	if self.vert then
		if self.move then
			if self.y > self.__y + self.dist * self.ts then
				self.move = false
			end
			self.yvLimit = 60
		else
			if self.y < self.__y - self.dist * self.ts then
				self.move = true
			end
			self.yvLimit = -60
		end
	else
		if self.move then
			if self.x > self.__x + self.dist * self.ts then
				self.move = false
			end
			self.xvLimit = 60
		else
			if self.x < self.__x - self.dist * self.ts then
				self.move = true
			end
			self.xvLimit = -60
		end
	end

	self:accelerate(dt)
	self:colide(dt)
end

function Platform:draw(s)
	love.graphics.draw(self.batch, self.x, self.y, 0, self.s)
end

return class("Platform", require("jnr.object"), Platform)
