local remove = table.remove
local pairs, pcall, tonumber, type, unpack = pairs, pcall, tonumber, type, unpack

local log = newLog "[PlayerParse]"

local function parsePlayerData(raw, allowVal)
	log "Parsing Player Data..."
	local data = {}

	for line in (raw.."\n"):gmatch("(.-)[\n\r]+") do
		local k, v = line:match("^(.-)[:=]=?(.*)$")
		local vn = tonumber(v)
		if vn then v = vn
		elseif v == "true" then v = true
		elseif v == "false" then v = false
		else
			local hasArray = v:find(",")
			if hasArray ~= nil then
				local array = {}
				for s in (v..","):gmatch("(.-),") do
					local sn = tonumber(s)
					if sn then s = sn
					elseif s == "true" then s = true
					elseif s == "false" then s = false
					end
					array[#array+1] = s
				end
				v = array
			end
		end
		data[k] = v
	end

	local out = {}

	if type(data.anim) == "table" then
		out.anim = anim.newHeader(assets.image("assets/char/"..tostring(data.anim[1])..".png"), tonumber(data.anim[2]) or 11, tonumber(data.anim[3]) or 19)
		data.anim = nil
	end

	if not allowVal then
		if data.xrOff and data.w then
			data.xrOff = floor(data.xrOff - (data.w - 8) * 0.5)
			log "\tNeeded to change 'xrOff'"
		end
		if data.yrOff and data.h then
			data.yrOff = floor(data.yrOff - (data.h -17) * 0.5)
			log "\tNeeded to change 'yrOff'"
		end
		for k,v in pairs {
			"w", "h", "pJumps",
			"normalJump", "wallJump", "airJump",
			"walkSpeed", "crawlSpeed", "wallSpeed",
			"swimHori", "swimVert"
		} do
			data[v] = nil
		end
	end

	for k,v in pairs(data) do
		local t = type(v)
		if t == "table" and out.anim then
			if #v == 5 then
				local x, y, len, spd, loop = v[1], v[2], v[3], v[4], v[5]
				if  type(x)   == "number" and type(y)   == "number"
				and type(len) == "number" and type(spd) == "number"
				and (loop == nil or type(loop) == "boolean") then
					out.anim:reg(k, x, y, len, spd, loop)
				else
					log("\tAnimation (REG) '"..k.."' invalid.")
				end
			else
				local loop = remove(v, #v)
				local spd  = remove(v, #v)
				local frms = { unpack(v) }
				local validfrms = true
				for i=1, #frms do if type(frms[i]) ~= "number" then validfrms = false break end end
				if  validfrms and type(spd) == "number"
				and (loop == nil or type(loop) == "boolean") then
					out.anim:spc(k, frms, spd, loop)
				else
					log("\tAnimation (SPC) '"..k.."' invalid.")
				end
			end
		elseif t == "number" then
			out[k] = v
		else
			log("\tData '"..k.."' is not a number or animation.")
		end
	end

	return out
end

return parsePlayerData
