local roundn = math.roundn
local pairs = pairs

local SceneTrigger = {}

function SceneTrigger:new(x, y, z, w, h, scene, onGroundOnly, arg)
	self:__new(x, y, 1, 1, -127)

	self.__x = x
	self.__y = y
	self.__w = w
	self.__h = h

	self.onGroundOnly = onGroundOnly or false

	self.canColide = false
	self.mass = 1

	self.xvAcc = 0.2
	self.yvAcc = 0.2
	self.yvLimit = 0

	self.scene = scene

	if arg then
		if arg.f then self.setFlag = arg.f end
		if arg.j then self.inJnr   = arg.j end
		self.arg = arg
	end
end

function SceneTrigger:init()
	local ts = self.tiles.ts
	self.x = self.__x*ts-ts
	self.y = self.__y*ts-ts
	self.w = self.__w*ts
	self.h = self.__h*ts
end

function SceneTrigger:hasColided(other, dir, dt)
	if other.type == "Player" and not other.dead then
		if other.data.ground or not self.onGroundOnly then
			if self.setFlag then save.data.flags[self.setFlag] = true end
			self:remove()
			if self.inJnr then
				jnr.triggerScene(self.scene, self.arg)
			else
				game.setState("scene", self.scene, self.arg)
			end
		end
	end
end

-- Do nothing!
function SceneTrigger:update(dt) end
function SceneTrigger:draw(s) end

function SceneTrigger:__tostring()
	return ("SceneTrigger: Scene %s"):format(self.scene or "unset(?)")
end

return class("SceneTrigger", require("jnr.object"), SceneTrigger)
