local love = love
local abs, roundn, pi = math.abs, math.roundn, math.pi
local colors = colors
local sfx = sfx

local Switch = {}

Switch.directions = {
	up = { -0.75*pi, -0.25*pi, -0.5*pi },
	down = { 0.25*pi, 0.75*pi, 0.5*pi },
	left = { 0.75*pi, 1.25*pi, pi },
	right = { -0.25*pi, 0.25*pi, 0 },
}

function Switch:new(x, y, z, facing, ...)
	self:__new(x, y, 9, 9, -2)

	self.canColide = true
	self.mass = 9e9

	self.xvAcc = 0.2
	self.yvAcc = 0.2
	self.yvLimit = 0

	self.active = false
	if ... then
		self.__arg = {...}
	end

	self.facing = facing
	self.__limits = self.directions[facing]
	self.switch = self.__limits[1]
	self.base = self.__limits[3]

	self.deleting = false
	self.deleteWho = false

	self.image = assets.image("assets/object/switch.png")
	self.quadBase = love.graphics.newQuad(0, 0, 6, 9, 15, 9)
	self.quadTop  = love.graphics.newQuad(7, 0, 8, 9, 15, 9)

	self.data.ground = true
end

function Switch:init()
	local v = self.__arg
	if v then
		local toDelete
		if v[1] == "Entity" then
			toDelete = Entity(v[2], v[3], -1, jnr.getEntityData(v[4]), v[5], v[6], v[7], v[8], v[9])
		else
			toDelete = (jnr[v[1]] or v[1])(v[2], v[3], -1, v[4], v[5], v[6], v[7], v[8], v[9])
		end
		self.tiles:addObject(toDelete)
		self.toDelete = toDelete
	end
end

function Switch:update(dt)
	if self.active and self.switch < self.__limits[2] then
		self.switch = self.switch + dt * 16
		if self.switch > self.__limits[2] then self.switch = self.__limits[2] end
	elseif not self.active and self.switch > self.__limits[1] then
		self.switch = self.switch - dt * 16
		if self.switch < self.__limits[1] then self.switch = self.__limits[1] end
	end

	if self.deleting then
		self.deleting = self.deleting + dt
		if self.deleteWho and self.deleting > 0.5 then
			self.deleteWho:kill()
			self.deleteWho = nil
		elseif self.deleting > 1 then
			jnr.scrollOverride()
			self.deleting = false
		end
	end
end

function Switch:hurt(who)
	if who.type == "Player" then
		local vol = (self.xv*self.xv+self.yv*self.yv)^0.5 * 0.0016
		sfx.colide:play(self.x + self.w*0.5, self.y + self.h*0.5, vol)

		self.active = not self.active
		if self.active and self.toDelete then
			self.deleting = 0
			self.deleteWho = self.toDelete

			jnr.scrollOverride(
				self.toDelete.x + self.toDelete.w*0.5,
				self.toDelete.y + self.toDelete.h*0.5,
				3
			)

			self.toDelete = nil
		end
	end
	self.xv = 0
	self.yv = 0
end

function Switch:hasColided()
	self.xv = 0
	self.yv = 0
end

function Switch:draw(s)
	local x, y = roundn(self.x, s)+4.5, roundn(self.y, s)+4.5

	love.graphics.draw(self.image, self.quadTop, x, y, self.switch, 1, 1, 1.5, 4.5)
	love.graphics.draw(self.image, self.quadBase, x, y, self.base, 1, 1, 4.5, 4.5)
end

function Switch:drawL(s)
	local del = self.toDelete or self.deleteWho
	if del then
		local x, y = roundn(self.x, s)+4.5, roundn(self.y, s)+4.5
		local x2, y2 = roundn(del.x + del.w*0.5, s), roundn(del.y + del.h*0.5, s)

		love.graphics.setColor(colors.red)
		if x == x2 or y == y2 then
			love.graphics.line(x2, y2, x, y)
		elseif abs(x2-x) < abs(y2-y) then
			love.graphics.line(x2, y2, x, y2, x, y)
		else
			love.graphics.line(x2, y2, x2, y, x, y)
		end

		love.graphics.rectangle("fill", x-1.5, y-1.5, 3, 3)
		love.graphics.rectangle("fill", x2-1.5, y2-1.5, 3, 3)

		love.graphics.setColor(colors.white)
	end
end

function Switch:__tostring()
	return ("Switch: %s; %s"):format(self.active and "On" or "Off", tostring(self.toDelete or "(lol)"))
end

return class("Switch", require("jnr.object"), Switch)
