local love = love
local abs, roundn, sin, cos, pi = math.abs, math.roundn, math.sin, math.cos, math.pi
local input, colors = input, colors
local sfx = sfx

local WarpDoor = {}

do
	local w, h = 13, 27
	local iw, ih = 52, 54
	WarpDoor.parts = {
		love.graphics.newQuad( 0, 0, w, h, iw, ih),
		love.graphics.newQuad(13, 0, w, h, iw, ih),
		love.graphics.newQuad(26, 0, w, h, iw, ih),
		love.graphics.newQuad(39, 0, w, h, iw, ih),
		love.graphics.newQuad( 0, h, w, h, iw, ih),
		love.graphics.newQuad(13, h, w, h, iw, ih),
		love.graphics.newQuad(26, h, w, h, iw, ih),
		love.graphics.newQuad(39, h, w, h, iw, ih),
	}

	local iw, ih = 52, 27
	WarpDoor.partsSp = {
		love.graphics.newQuad( 0, 0, w, h, iw, ih),
		love.graphics.newQuad(13, 0, w, h, iw, ih),
		love.graphics.newQuad(26, 0, w, h, iw, ih),
		love.graphics.newQuad(39, 0, w, h, iw, ih),
		love.graphics.newQuad(39, 0, w, h, iw, ih),
		love.graphics.newQuad(39, 0, w, h, iw, ih),
	}
end

function WarpDoor:new(x, y, z, goesTo)
	self:__new(x, y, 9, 18, -127)
	self.__x = x
	self.__y = y

	self.canColide = false
	self.mass = 150

	self.xvAcc = 0.2
	self.yvAcc = 0.2
	self.yvLimit = 0

	self.close = 0

	if type(goesTo) == "table" then
		local levelName = goesTo.name
		self.goesTo = goesTo.hub

		self.text = levelName

		if goesTo.door then
			self.image = assets.image("assets/object/warp_door_sp.png")
			self.quad = self.partsSp[goesTo.door]
		else
			self.image = assets.image("assets/object/warp_door.png")
			self.quad = self.parts[3]
		end
	else
		local maxCollect = jnr.collect[goesTo]
		local finished = save.data.flags[goesTo]
		local allCollect = false

		local levelName = fileT.require("jnr/level/#names.txt")[goesTo] or goesTo
		self.goesTo = goesTo

		if maxCollect then
			local collect = save.data.collect[goesTo] or 0
			allCollect = collect >= maxCollect
			self.text = {
				finished and colors.pGreen or colors.pRed, levelName,
				colors.white, "\n["..tostring(collect).."/"..tostring(maxCollect).."£]"
			}
		else
			self.text = levelName
		end

		self.image = assets.image("assets/object/warp_door.png")
		local part = not maxCollect and 1 or not finished and 2 or allCollect and 4 or 3
		self.stateOpened = self.parts[part]
		self.stateClosed = self.parts[part+4]
		self.quad = self.stateOpened
	end
end

function WarpDoor:update(dt)
	if input.new("attack") then
		if self:overlap(jnr.player) then
			if jnr.inHub then
				save.data.coord = { self.__x, self.__y, jnr.player.dir }
			end
			game.setState("jnr", self.goesTo)
			input.releaseInput("attack")
		end
	end

	if self.canColide then
		self:setAcceleration()
		self:accelerate(dt)
		self:colide(dt)
		self:spawnParticles(dt)
		self:makeSounds()

		if self.close > 0 then
			self.close = self.close - dt * 3
			if self.close < 0 then self.close = 0 end
		end

		if self.y > self.tiles.height * self.tiles.ts then
			self:respawn()
		end
	else
		local distance = ((self.x-jnr.player.x)^2 + (self.y-jnr.player.y)^2)^0.5
		if distance < 65 then
			local b = distance > 23 and 1-(distance-23)/42 or 1
			if self.close < b then
				self.close = self.close + dt * 3
				if self.close > b then self.close = b end
			elseif self.close > b then
				self.close = self.close - dt * 3
				if self.close < b then self.close = b end
			end
		elseif self.close > 0 then
			self.close = self.close - dt * 3
			if self.close < 0 then self.close = 0 end
		end
	end
end

function WarpDoor:hurt(who)
	if who.type == "Player" then
		if not self:overlap(who) and self.stateOpened and self.stateClosed then
			local vol = (self.xv*self.xv+self.yv*self.yv)^0.5 * 0.0016
			sfx.colide:play(self.x + self.w*0.5, self.y + self.h*0.5, vol)
			self.tiles.particle:spawn("smoke", 17, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)

			self.canColide = not self.canColide
			if self.canColide then
				self.quad = self.stateClosed
			else
				self.quad = self.stateOpened
			end
		end
	else
		self:respawn()
	end
	self.xv = 0
	self.yv = 0
end

function WarpDoor:hasColided(other, dir, dt)
	if dir == "n" and self.canColide and other.canColide then
		other.y = other.y - dt * 10
		other.yv = 0
		other.data.ground = true
	end
end

function WarpDoor:respawn()
	self.tiles.particle:spawn("smoke", 30, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
	self:reset()
	self.tiles.particle:spawn("smoke", 30, self.x + self.w * 0.5, self.y + self.h * 0.5, self.w*0.5, self.h*0.5)
end

function WarpDoor:draw(s)
	love.graphics.draw(self.image, self.quad, roundn(self.x + 4, s), roundn(self.y + 14, s), 0, 1, 1, 6, 18)
end

function WarpDoor:drawL(s)
	if self.close > 0 then
		local x = roundn(self.x, s)
		local y = roundn(self.y, s)

		local prompt = input.getPrompt("attack")
		love.graphics.setColor(colors.black(1, 1, 1, self.close))
		love.graphics.printf(prompt, x - 99, y - 11, 209, "center")
		love.graphics.print(self.text, x + self.w + 3, y - 1)

		love.graphics.setColor(colors.yellow(1, 1, 1, self.close))
		love.graphics.printf(prompt, x - 100, y - 12, 209, "center")

		love.graphics.setColor(colors.white(1, 1, 1, self.close))
		love.graphics.print(self.text, x + self.w + 2, y - 2)

		love.graphics.setColor(colors.white)
	end
end

function WarpDoor:__tostring()
	return ("WarpDoor: To %s"):format(self.goesTo or "unset(?)")
end

return class("WarpDoor", require("jnr.object"), WarpDoor)
