-- Uh, too long of a list of functions... Just read the code and try to get it.
-- Dependencies:
local ceil, floor, max, min = math.ceil, math.floor, math.max, math.min
local tostring, tonumber, type, ipairs, assert = tostring, tonumber, type, ipairs, assert
local setmetatable = setmetatable
local newQuad, draw = love.graphics.newQuad, love.graphics.draw

-- Module:
local anim2 = {}

--[==[
Regular Animation Header (Single Simultaneous Animation):
]==]
local header = {}
header.__index = header

function header:reg(id, tx, ty, l, sp, loop)
	assert(id ~= nil, "Animation ID may not be nil!")
	if loop == nil then loop = false end

	local lp
	if type(loop) == "number" then
		lp = (loop - 1) / sp
		loop = true
	end

	local frames = {}
	local new = {
		s = sp, l = l, r = loop, p = lp,
		f = frames
	}

	local i = 0
	while i < l do
		frames[i] = self:getQuad(tx, ty)

		tx = tx + 1
		if tx > self.max then
			tx = 1
			ty = ty + 1
		end

		i = i + 1
	end

	self.anims[id] = new

	return self
end

function header:spc(id, all, sp, loop)
	assert(id ~= nil, "Animation ID may not be nil!")
	if loop == nil then loop = false end

	local lp
	if type(loop) == "number" then
		lp = (loop - 1) / sp
		loop = true
	end
	local l = floor(#all/2)

	local frames = {}
	local new = {
		s = sp, l = l, r = loop, p = lp,
		f = frames
	}

	local i = 0
	while i < l do
		local tx, ty = all[i*2+1], all[i*2+2]

		frames[i] = self:getQuad(tx, ty)

		i = i + 1
	end

	self.anims[id] = new

	return self
end

function header:copy(orig, new)
	self.anims[new] = self.anims[orig]

	return self
end

function header:getQuad(tx, ty)
	local id = tx + (ty-1)*self.max
	if self.sheet[id] then
		return self.sheet[id]
	else
		local q = newQuad((tx-1)*(self.tx+self.spacing), (ty-1)*(self.ty+self.spacing), self.tx, self.ty, self.img:getDimensions())
		self.sheet[id] = q
		return q
	end
end

function header:setQuad(tx, ty, w, h)
	local id = tx + (ty-1)*self.max

	self.sheet[id] = newQuad((tx-1)*(self.tx+self.spacing), (ty-1)*(self.ty+self.spacing), w*self.tx, h*self.ty, self.img:getDimensions())

	return self
end

function header:set(next)
	if self.anim ~= next then
		self.anim = next
		self.time = 0
		self.loops = 0
		self.dead = true
	end

	return self
end

function header:reset()
	self.time = 0
	self.loops = 0
	return self
end

function header:update(dt)
	local anim = self.anims[self.anim]
	if anim then
		if self.dead then
			self.dead = false
		else
			self.time = self.time + dt
			local f = floor(self.time * anim.s)

			if f > anim.l - 1 then
				if anim.r then
					self.time = self.time % (anim.l / anim.s)
					if anim.p then self.time = self.time + anim.p end
					f = floor(self.time * anim.s)
					self.loops = self.loops + 1
				else
					if anim.f[anim.l - 1] then
						self.frame = anim.f[anim.l - 1]
					end
					return true
				end
			end

			if anim.f[f] then
				self.frame = anim.f[f]
			end
		end
		return false
	else
		return true
	end
end

function header:draw(x, y, r, sx, sy, ox, oy, kx, ky)
	draw(self.img, self.frame, x, y, r, sx, sy, ox, oy, kx, ky)
end

function header:__tostring()
	if self.__address then
		return ("AnimHeader: 0x%x"):format(self.__address)
	else
		return "AnimHeader: nil"
	end
end

function header:clone()
	-- All cloned headers and the original are interlinked!
	-- Animations and the Sheet will be shared, no matter what!
	local new = {
		img = self.img, tx = self.tx, ty = self.ty, max = self.max,
		anims = self.anims, sheet = self.sheet, spacing = self.spacing,
		anim = nil, time = 0, loops = 0, dead = false
	}
	new.__address = tonumber(tostring(new):match("^table: 0x([0-9a-f]*)$"), 16)

	setmetatable(new, header)
	new.frame = new:getQuad(1, 1)

	return new
end

-- Constructor:
function anim2.newHeader(img, tx, ty, arg)
	local max = floor(img:getWidth()/tx)

	local new = {
		img = img, tx = tx, ty = ty, max = max,
		anims = {}, sheet = {}, spacing = 0,
		anim = nil, time = 0, loops = 0, dead = false
	}
	new.__address = tonumber(tostring(new):match("^table: 0x([0-9a-f]*)$"), 16)

	if arg then
		if arg.spacing then
			new.spacing = arg.spacing
			new.max = floor((img:getWidth()+new.spacing)/(tx+new.spacing))
		end

		if arg.allowmirror then
			new.max = new.max * 2
			img:setWrap("mirroredrepeat", "mirroredrepeat")
		end
	end

	setmetatable(new, header)
	new.frame = new:getQuad(1, 1)

	return new
end

--[==[
Multi-Animation Header (Multiple Simultaneous Animations):
Useful for animated tile sets.
]==]
local mheader = {}
mheader.__index = mheader

function mheader:reg(tx, ty, l, sp)
	local frames = {}
	local new = {
		s = sp, l = l,
		f = frames, t = 0
	}

	local i = 0
	while i < l do
		frames[i] = self:getQuad(tx, ty)

		tx = tx + 1
		if tx > self.max then
			tx = 1
			ty = ty + 1
		end

		i = i + 1
	end

	new.quad = new.f[1]
	self.anims[#self.anims+1] = new

	return new
end

function mheader:spc(all, sp)
	local l = floor(#all/2)

	local frames = {}
	local new = {
		s = sp, l = l,
		f = frames, t = 0
	}

	local i = 0
	while i < l do
		local tx, ty = all[i*2+1], all[i*2+2]

		frames[i] = self:getQuad(tx, ty)

		i = i + 1
	end

	new.quad = new.f[1]
	self.anims[#self.anims+1] = new

	return new
end

mheader.getQuad = header.getQuad

function mheader:update(dt)
	for i,anim in ipairs(self.anims) do
		anim.t = anim.t + dt
		local f = floor(anim.t * anim.s)

		if f > anim.l - 1 then
			anim.t = anim.t % (anim.l / anim.s)
			f = floor(anim.t * anim.s)
		end

		if anim.f[f] then
			anim.quad = anim.f[f]
		end
	end
end

function mheader:__tostring()
	if self.__address then
		return ("MultiHeader: 0x%x"):format(self.__address)
	else
		return "MultiHeader: nil"
	end
end

-- Constructor:
function anim2.newMultiHeader(img, tx, ty, arg)
	local max = floor(img:getWidth()/tx)

	local new = {
		img = img, tx = tx, ty = ty, max = max,
		anims = {}, sheet = {}, spacing = 0,
	}
	new.__address = tonumber(tostring(new):match("^table: 0x([0-9a-f]*)$"), 16)

	if arg then
		if arg.spacing then
			new.spacing = arg.spacing
			new.max = floor((img:getWidth()+new.spacing)/(tx+new.spacing))
		end

		if arg.allowmirror then
			new.max = new.max * 2
			img:setWrap("mirroredrepeat", "mirroredrepeat")
		end
	end

	setmetatable(new, mheader)

	return new
end

return anim2
