-- Don't use this! There's no better alternatives!
local pairs, unpack = pairs, unpack
local love = love

local input = {}

local binds = {}
local bindsB = {}
local bindsA = {}
local events = {}

function input.bindKey(scancode, id)
	binds[scancode] = id
end

function input.bindButton(button, id)
	bindsB[button] = id
end

function input.bindAxis(axis, thres, idl, idu)
	bindsA[axis] = {
		t = thres,
		neg = idl, pos = idu
	}
end

function input.bindEvent(event, id)
	events[id] = event
end

function input.getBoundKey(id)
	local all = {}
	for k,v in pairs(binds) do
		if v == id then
			all[#all+1] = k
		end
	end
	return unpack(all)
end

function input.getBoundButton(id)
	local all = {}
	for k,v in pairs(bindsB) do
		if v == id then
			all[#all+1] = k
		end
	end
	return unpack(all)
end

function input.getBoundAxis(id)
	local all = {}
	for k,v in pairs(bindsA) do
		if v.neg == id then
			all[#all+1] = k
			all[#all+1] = -1
		elseif v.pos == id then
			all[#all+1] = k
			all[#all+1] = 1
		end
	end
	return unpack(all)
end

function input.isKeyBound(scancode) return binds[scancode] and true or false end
function input.isButtonBound(button) return bindsB[button] and true or false end
function input.isAxisBound(axis) return bindsA[axis] and true or false end

local separated, bindButton, bindAxis, getLastType
function input.separate(state)
	if separated == nil then separated = false end
	if state ~= false and state ~= true then state = true end
	if state ~= separated then
		if state then
			bindButton, bindAxis, getLastType = input.bindButton, input.bindAxis, input.getLastType
			function input.bindButton(button, id)
				bindButton(button, id.."-gp")
			end
			function input.bindAxis(axis, thres, idl, idu)
				bindAxis(axis, thres, idl.."-gp", idu.."-gp")
			end
			function input.getLastType() return "keyboard" end

			local B = bindsB bindsB = {}
			for k,v in pairs(B) do bindsB[k] = v.."-gp" end
			local A = bindsA bindsA = {}
			for k,v in pairs(A) do
				bindsA[k] = {
					t = v.t,
					neg = v.neg.."-gp", pos = v.pos.."-gp"
				}
			end
		else
			input.bindButton, input.bindAxis, input.getLastType = bindButton, bindAxis, getLastType

			local B = bindsB bindsB = {}
			for k,v in pairs(B) do bindsB[k] = v:gsub("%-gp$", "") end
			local A = bindsA bindsA = {}
			for k,v in pairs(A) do
				bindsA[k] = {
					t = v.t,
					neg = v.neg:gsub("%-gp$", ""), pos = v.pos:gsub("%-gp$", "")
				}
			end
		end
		separated = state
	end
end

local held = {}
local sour = {}
local tap  = {}
local rep  = {}
local axes = {}

function input.down(id)
	return held[id] ~= nil
end

function input.new(id)
	return tap[id] or false
end

function input.rep(id)
	return tap[id] or rep[id] or false
end

function input.axis(id)
	return axes[id] or 0
end

function input.hitInput(id)
	tap [id] = true
end

function input.holdInput(id)
	held[id] = 0
	tap [id] = true
end

function input.releaseInput(id)
	held[id] = nil
	tap [id] = false
end

local lastType

local function keypressed(key, scancode, isrepeat)
	if not isrepeat then
		local b = binds[scancode]
		if b and not held[b] then
			lastType = "keyboard"

			held[b] = 0
			sour[b] = "key"
			tap [b] = true

			if events[b] then events[b]() end
		end
	end
end

local function keyreleased(key, scancode)
	local b = binds[scancode]
	if b and sour[b] == "key" then
		held[b] = nil
	end
end

local function gamepadpressed(joystick, button)
	local b = bindsB[button]
	if b and not held[b] then
		lastType = "gamepad"

		held[b] = 0
		sour[b] = "button"
		tap [b] = true

		if events[b] then events[b]() end
	end
end

local function gamepadreleased(joystick, button)
	local b = bindsB[button]
	if b and sour[b] == "button" then
		held[b] = nil
	end
end

local axisPos = setmetatable({}, {
	__index = function() return 0 end
})

local function gamepadaxis(joystick, axis, value)
	local b = bindsA[axis]
	if b then
		local ov = axisPos[axis]

		if value > ov then
			if not held[b.pos] and value >= b.t and ov < b.t then
				held[b.pos] = 0
				sour[b.pos] = "axis"
				tap [b.pos] = true

				if events[b.pos] then events[b.pos]() end

				if sour[b.neg] == "axis" then
					held[b.neg] = nil
				end
			elseif sour[b.neg] == "axis" and value > -b.t and ov <= -b.t then
				held[b.neg] = nil
			end
		elseif value < ov then
			if not held[b.neg] and value <= -b.t and ov > -b.t then
				held[b.neg] = 0
				sour[b.neg] = "axis"
				tap [b.neg] = true

				if events[b.neg] then events[b.neg]() end

				if sour[b.pos] == "axis" then
					held[b.pos] = nil
				end
			elseif sour[b.pos] == "axis" and value < b.t and ov >= b.t then
				held[b.pos] = nil
			end
		end

		if value >= b.t then
			lastType = "gamepad"

			axes[b.pos] = (value-b.t)/(1-b.t)
			axes[b.neg] = 0
		elseif value <= -b.t then
			lastType = "gamepad"

			axes[b.pos] = 0
			axes[b.neg] = (-value-b.t)/(1-b.t)
		else
			axes[b.pos] = 0
			axes[b.neg] = 0
		end
	end

	axisPos[axis] = value
end

input.keypressed      = keypressed
input.keyreleased     = keyreleased
input.gamepadpressed  = gamepadpressed
input.gamepadreleased = gamepadreleased
input.gamepadaxis     = gamepadaxis

function input.overrideCallbacks()
	love.keypressed      = keypressed
	love.keyreleased     = keyreleased
	love.gamepadpressed  = gamepadpressed
	love.gamepadreleased = gamepadreleased
	love.gamepadaxis     = gamepadaxis
end

function input.hookCallbacks()
	if love.keypressed then
		local lkeypressed = love.keypressed
		function love.keypressed(key, scancode, isrepeat)
			lkeypressed(key, scancode, isrepeat)
			keypressed(key, scancode, isrepeat)
		end
	else
		love.keypressed = keypressed
	end

	if love.keyreleased then
		local lkeyreleased = love.keyreleased
		function love.keyreleased(key, scancode)
			lkeyreleased(key, scancode)
			keyreleased(key, scancode)
		end
	else
		love.keyreleased = keyreleased
	end

	if love.gamepadpressed then
		local lgamepadpressed = love.gamepadpressed
		function love.gamepadpressed(joystick, button)
			lgamepadpressed(joystick, button)
			gamepadpressed(joystick, button)
		end
	else
		love.gamepadpressed = gamepadpressed
	end

	if love.gamepadreleased then
		local lgamepadreleased = love.gamepadreleased
		function love.gamepadreleased(joystick, button)
			lgamepadreleased(joystick, button)
			gamepadreleased(joystick, button)
		end
	else
		love.gamepadreleased = gamepadreleased
	end

	if love.gamepadaxis then
		local gamepadaxis = love.gamepadaxis
		function love.gamepadaxis(joystick, axis, value)
			lgamepadaxis(joystick, axis, value)
			gamepadaxis(joystick, axis, value)
		end
	else
		love.gamepadaxis = gamepadaxis
	end
end

function input.update(dt)
	tap = {}
	rep = {}

	for k,v in pairs(held) do
		v = v + dt
		if v > 0.5 then
			v = v % 0.1 + 0.4
			rep[k] = true
		end
		held[k] = v
	end
end

local keyNames = setmetatable({
	space = "Space",
	kp0 = "Num0",
	kp1 = "Num1", kp2 = "Num2", kp3 = "Num3",
	kp4 = "Num4", kp5 = "Num5", kp6 = "Num6",
	kp7 = "Num7", kp8 = "Num8", kp9 = "Num9",
	["kp."] = "Num.", ["kp,"] = "Num,", ["kp/"] = "Num/",
	["kp*"] = "Num*", ["kp-"] = "Num-", ["kp+"] = "Num+",
	kpenter = "NumEnter", ["kp="] = "Num=",
	up = "Up", down = "Down",
	left = "Left", right = "Right",
	home = "Home", ["end"] = "End",
	pageup = "Page Up", pagedown = "Page Down",
	insert = "Insert", delete = "Delete",
	backspace = "Backspace", ["return"] = "Return", tab = "Tab",
	clear = "Clear",
	numlock = "Num-Lock", capslock = "Caps-Lock", scrolllock = "Scroll-Lock",
	rshift = "R.Shift", lshift = "L.Shift",
	rctrl = "R.Ctrl", lctrl = "L.Ctrl",
	ralt = "R.Alt", lalt = "L.Alt",
	rgui = "R.Cmd", lgui = "L.Cmd",
	mode = "Mode",
	--www,mail,calculator,computer,appsearch,apphome,appback,appforward,apprefresh,appbookmarks
	pause = "Pause",
	escape = "Esc.",
	help = "Help",
	printscreen = "Print",
	--sysreq
	menu = "Menu",
	--application,power
	currencyunit = "$",
	undo = "Undo",
	unknown = "*???*"
}, {
	__index = function(self, key)
		local name = key:upper()
		self[key] = name
		return name
	end
})
input.keyNames = keyNames

local getKeyFromScancode = love.keyboard.getKeyFromScancode

function input.getKeyName(scancode)
	return keyNames[getKeyFromScancode(scancode)]
end

local buttonNames = {
	xinput = {
		a = "\226\150\160", b = "\226\150\161", x = "\226\150\162", y = "\226\150\163",
		start = "\226\150\164", back = "\226\150\165", guide = "Guide",
		rightstick = "RS", leftstick = "RS",
		rightshoulder = "RB", leftshoulder = "LB",
		dpleft = "\226\150\176", dpright = "\226\150\178",
		dpup = "\226\150\177", dpdown = "\226\150\179"
	},
	ds4 = {
		a = "\226\150\170", b = "\226\150\171", x = "\226\150\172", y = "\226\150\173",
		start = "Options", back = "Share", guide = "PS",
		rightstick = "R3", leftstick = "L3",
		rightshoulder = "R1", leftshoulder = "L1",
		dpleft = "\226\150\176", dpright = "\226\150\178",
		dpup = "\226\150\177", dpdown = "\226\150\179"
	}
}
input.buttonNames = buttonNames

function input.getButtonName(button, controller)
	return buttonNames[controller or "xinput"][button] or "None"
end

function input.getLastType()
	return lastType
end

do
	input.record = {}

	local data, restore
	local stream, alt, frame

	local function restoreData()
		if love.timer then
			love.timer.getDelta = restore.timer_getDelta
			love.timer.getAverageDelta = restore.timer_getAverageDelta
			love.timer.getFPS = restore.timer_getFPS
		end

		love.update = restore.love_update
	end

	function input.record.start(fps, consider, rng)
		if input.record.state ~= nil then return end
		input.record.state = "recording"

		data = {}
		restore = {}
		stream = {}
		alt = {}
		local state = {}
		local astat = {}

		frame = 0

		data.stream = stream
		data.alt = alt

		data.fps = fps
		data.consider = consider

		local dt = 1 / fps

		if love.timer then
			restore.timer_getDelta = love.timer.getDelta
			restore.timer_getAverageDelta = love.timer.getAverageDelta
			restore.timer_getFPS = love.timer.getFPS

			function love.timer.getDelta()
				return dt
			end

			function love.timer.getAverageDelta()
				return dt
			end

			function love.timer.getFPS()
				return fps
			end
		end

		if love.math then
			data.seed = os.time()
			love.math.setRandomSeed(data.seed)
		end

		restore.love_update = love.update
		function love.update()
			frame = frame + 1

			for i=1, #consider do
				local v = consider[i]
				local isDown = held[v] ~= nil
				if state[v] ~= isDown then
					state[v] = isDown
					if not stream[frame] then stream[frame] = {} end
					stream[frame][i] = isDown
				end
			end

			for i=1, #consider do
				local v = consider[i]
				local axis = axes[v] or 0
				if astat[v] ~= axis then
					astat[v] = axis
					if not alt[frame] then alt[frame] = {} end
					alt[frame][i] = axis
				end
			end

			restore.love_update(dt)
		end

		if rng then
			data.rng = {}
			for k,v in pairs(rng) do
				data.rng[k] = v:getState()
			end
		end

		print("Now recording your every move...")
	end

	function input.record.finish(write, to)
		if input.record.state ~= "recording" then return end
		input.record.state = nil

		restoreData()

		data.length = frame

		print("Aaaand done! Recording finished!")
		return write(to, data)
	end

	function input.record.replay(read, from, rng)
		if input.record.state ~= nil then return end
		input.record.state = "replaying"

		data = read(from)
		restore = {}
		frame = 0

		local consider = data.consider

		local fps = data.fps
		local dt = 1 / fps

		if love.timer then
			restore.timer_getDelta = love.timer.getDelta
			restore.timer_getAverageDelta = love.timer.getAverageDelta
			restore.timer_getFPS = love.timer.getFPS

			function love.timer.getDelta()
				return dt
			end

			function love.timer.getAverageDelta()
				return dt
			end

			function love.timer.getFPS()
				return fps
			end
		end

		local stream, alt = data.stream, data.alt

		if love.math then
			love.math.setRandomSeed(data.seed)
		end

		restore.love_update = love.update
		function love.update()
			frame = frame + 1
			if stream[frame] then
				local all = stream[frame]
				for k,v in pairs(all) do
					k = consider[k]
					if v then
						held[k] = 0
						tap [k] = true

						if events[k] then events[k]() end
					else
						held[k] = nil
					end
				end
			end
			if alt[frame] then
				local all = alt[frame]
				for k,v in pairs(all) do
					axes[consider[k]] = v
				end
			end

			restore.love_update(dt)

			if frame > data.length then
				input.record.state = nil

				restoreData()
			end
		end

		if data.rng and rng then
			for k,v in pairs(rng) do
				v:setState(data.rng[k])
			end
		end
	end
end

return input
