-- Enabling Debug mode...
local objects = {
	{ o=require"jnr.object", 9, 9, -2 },
	{ o=require"jnr.object", 18, 18, -2 },
	{ o=require"jnr.player", 0, jnr.getPlayerData "denniz" },
	{ o=require"jnr.check_point", -1 },
	{ o=require"jnr.barrier", -1, true, 3 },
	{ o=require"jnr.platform", -1, 3, 3, false, false },
	{ o=require"jnr.platform", -1, 3, 3, false, true },
}
local objsel = 1
local held

function love.mousepressed(x, y, button, isTouch)
	if game.state == jnr then
		x = (x / window.s + jnr.scroll.x - window.vw * 0.5)
		y = (y / window.s + jnr.scroll.y - window.vh * 0.5)

		if button == 1 then
			for k,v in pairs(jnr.tiles.object) do
				if v:overlap{x=x, y=y, w=1, h=1} then
					held = {v, x-v.x, y-v.y}
				end
			end
		elseif button == 2 then
			local ts = jnr.tiles.ts
			local n = objects[objsel].o(math.round(x/ts), math.round(y/ts), unpack(objects[objsel]))
			jnr.tiles:addObject(n)
			n.x = x
			n.y = y
		end
	end
end

function love.wheelmoved(x, y)
	objsel = objsel + y
	if objsel < 1 then
		objsel = #objects
	elseif objsel > #objects then
		objsel = 1
	end
end

function love.mousemoved(x, y, dx, dy)
	if held then
		if game.state == jnr then
			x = (x / window.s + jnr.scroll.x - window.vw * 0.5)
			y = (y / window.s + jnr.scroll.y - window.vh * 0.5)

			held[1].x = x - held[2]
			held[1].y = y - held[3]
			held[1].xv = 0
			held[1].yv = 0
		else
			held = nil
		end
	end
end

function love.mousereleased(x, y, button, isTouch)
	if held and game.state == jnr then
		held = nil
	end
end

local s,r = pcall(function()
	_debug, build, B, U = false, false, false, false

	-- Load the debugger utility and set it up
	local debugger = require "debugger"()
	debugger.allowFunctionIndex(true)
	debugger.monitorGlobal()

	-- Setting a larger version of the font for easier readability
	local FONT = love.graphics.newFont("assets/font.ttf", 15)
	FONT:setFilter("linear", "nearest")
	local FONTADD = love.graphics.newFont("assets/font_add.ttf", 15)
	FONTADD:setFilter("linear", "nearest")
	FONT:setFallbacks(FONTADD)
	debugger.setFont(FONT)

	debugger.doTempPrint = false

	debugger.newCommand("music", "s", music.set)
	debugger.newCommand("speed", "n", game.setSpeed)

	debugger.newCommand("flag", "s", function(f)
		save.data.flags[f] = not save.data.flags[f]
		return ":Set flag "..f.." to "..tostring(save.data.flags[f]).."."
	end)
	debugger.newCommand("flag", "sb", function(f, b)
		save.data.flags[f] = b
		return ":Set flag "..f.." to "..tostring(b).."."
	end)

	debugger.newCommand("store", "", function()
		local to = "screenlevel/"..jnr.currentLevel..(save.data.hardcore and "#uhc" or "")
		jnr.tiles:storeImage(to)
		return ":Saved image to "..to..".png"
	end)
	debugger.newCommand("store", "s", function(to)
		jnr.tiles:storeImage(to)
		return ":Saved image to "..to..".png"
	end)

	local colorAlpha = 1
	debugger.newCommand("alpha", "n", function(new)
		if new == 0 then new = 1e-100 end
		for k,v in pairs(colors) do
			colors[k][4] = colors[k][4] * new/colorAlpha
		end
		colorAlpha = new
		return ":Updated global alpha multiplier"
	end)

	_debug = debugger
end)
if not s then print(r) end

input.bindKey("f9", "toggleDebugDisplay")
input.bindEvent(function()
	debugging = not debugging
	love.mouse.setVisible(debugging)
end, "toggleDebugDisplay")

build = require "build"
game.states.build = build
function B(...)
	game.setState("build", ...)
end

function U()
	local collect = {}
	local Tiles = require "jnr.tiles"

	local function load(v)
		local data = love.filesystem.load("jnr/level/"..v)()
		if data.collect then
			local id = v:gsub("%.[^%.]*$", "")
			local tiles = Tiles(data)
			collect[id] = tiles.maxCollect
		end
	end

	for i,v in ipairs(love.filesystem.getDirectoryItems("jnr/level")) do
		if not v:find("^#") and not v:find("#uhc$") then
			local s, e = pcall(load, v)
			if not s then print("ERR:", i, v, e) end
		end
	end

	local order = {}
	for k,v in pairs(collect) do order[#order+1] = k end
	table.sort(order)

	local s = ""
	for i,v in ipairs(order) do
		s = s .. tostring(v) .."=".. tostring(collect[v]) .."\n"
	end

	love.filesystem.write("#collect.txt", s)

	return collect
end
