local roundn = math.roundn
local window, colors = window, colors

local log = newLog "Characters"

local data
local select = 1

local function cancel(sel)
	if data.prevData then
		game.setState("menu", data.prev, menu.substate, true)
	else
		if sel == true then
			game.setState("jnr", jnr.reload)
			game.queueTask(game.message.queue, "Reloaded level to update the character.")
		else
			game.setState("jnr", game.noInit)
		end
	end
end

local allNames = fileT.load("jnr/char/#names.txt")
local allUnlck = fileT.load("jnr/char/#unlck.txt")

local num = 0
local function getChar(id, open)
	if unlocks[id] or open then
		num = num + 1
		if id == cnf.char then select = num end

		local s, playerData = pcall(function()
			return jnr.getPlayerData(id)
		end)

		if s then
			local name = playerData.name or id
			local desc = playerData.desc

			local anim = playerData.anim
			if anim then
				anim = anim:clone()
				anim:set "idle"
			end

			return {
				text = name,
				desc = desc,
				action = function()
					if data.prevData then
						for i,v in ipairs(data.prevData) do if v then
							if type(v.text) == "string" then
								if v.text:find("^Change Character> ") then
									v.text = "Change Character> "..name
									break
								end
							end
						end end
					else
						game.queueTask(saveLib.write, "cnf", cnf)
					end

					cnf.char = id

					cancel(true)
				end,
				anim = anim, __data = playerData
			}
		else
			log("Error loading character in selection menu:", playerData)

			return { text = "Cannot load '"..tostring(id).."'!" }
		end
	elseif open == false then
		return nil
	else
		num = num + 1
		return {
			text = "LOCKED", desc = allUnlck[id] and {
				colors.white, "Locked character.\nHint: ",
				colors.orange, allUnlck[id]
			} or "Locked character."
		}
	end
end

local lastselect

data = {
	label = "Character Select",
	color = colors.new(200, 100, 0),
	cancel = cancel,
	update = function(self, dt, select)
		if lastselect ~= select then
			if data[lastselect] then if data[lastselect].anim then
				data[lastselect].anim:set "idle"
			end end
			if data[select] then if data[select].anim then
				data[select].anim:set "walk"
			end end

			lastselect = select
		end

		for i=1, #data do
			if data[i] then
				if data[i].anim then
					if select == i then
						data[i].anim:update(dt*0.65)
					else
						data[i].anim:update(dt)
					end
				end
			end
		end
	end,
	draw = function(self, by, select)
		for i=1, #data do
			if data[i] then
				if data[i].anim then
					local a = data[i].anim
					a:draw(roundn(window.vw*0.5 - 106 + data[i].offset - a.tx, window.s), roundn(by + i*16-9.5-a.ty*0.5, window.s))
				end
			end
		end
	end,

	getChar("denniz", true),
	getChar("diana", true),
	getChar "small_denniz",
	getChar("siblings", false),
	getChar("purple_sister", false),
	false,
	{
		text = "Back", action = cancel
	}
}

local insert, remove = table.insert, table.remove

if mods.__init then
	log("Loading modded characters!")
	for k,v in pairs(mods.chars) do
		insert(data, #data-1, getChar(v.id))
	end
end

local allFiles = love.filesystem.getDirectoryItems("jnr/char")
for i=1, #allFiles do
	local v = allFiles[i]
	if v:find("%.dpp2c$") then
		local id = v:match("^(.*)%.dpp2c$")
		insert(data, #data-1, getChar(id, true))
	end
end

for i=#data, 1, -1 do
	if data[i] == nil then
		remove(data, i)
	end
end

data.select = select

return data
