local colors = colors

local data = {}

local levelNames = fileT.require("jnr/level/#names.txt")
local accept

local function getButton(slot, select)
	local file = "save"..tostring(slot)
	if love.filesystem.isFile(file) then
		local data = saveLib.read(file)
		local ret
		if data then
			local time = save.formatTime(data.time or 0)

			local deaths = data.deaths ~= 1 and "s" or ""

			local collect = data.totalCollect or 0
			local totalCollect = jnr.collect.__total or 0/0
			local progress = 100*collect/totalCollect

			ret = {
				text = {
					colors.white, ("Del. Slot %s: [%s] [%d%%] "):format(slot, time, progress),
					data.hardcore and colors.uhc or colors.gold, data.hardcore and "[UHC]" or "[Normal]"
				},
				desc = {
					colors.white, ("Last seen at [%s].\nCollected [%d/%d (%d%%)] things...\nPlayed a total of [%s] hours.\nDied [%d] time%s in "):format(levelNames[data.hub] or data.hub, collect, totalCollect, progress, time, data.deaths or 0, deaths),
					data.hardcore and colors.uhc or colors.gold, data.hardcore and "Ultra Hardcore Mode" or "Normal Mode",
					colors.white, "."
				}
			}
		else
			ret = {
				text = {
					colors.red, ("Del. Slot %s: Corrupted?"):format(slot),
				},
				desc = "Corrupted data cannot be loaded. Delete it via the deletion menu."
			}
		end
		ret.action = function(self)
			if accept == select then
				love.filesystem.remove(file)
				self.text = "Data deleted."
				self.origText = ("Slot %s: Empty"):format(slot)
				self.desc = { colors.red, "No Data." }
				self.action = function() end
			else
				accept = select
				self.origText = self.text
				self.text = { colors.red, ("Really delete Slot %s?"):format(slot) }
			end
		end
		return ret
	else
		return {
			text = ("Slot %s: Empty"):format(slot),
			desc = { colors.red, "No Data." },
			action = function() end
		}
	end
end

local function cancel()
	if data.prev == "main" then
		local main = data.prevData

		main[1] = main.getButton("A")
		main[2] = main.getButton("B")
		main[3] = main.getButton("C")
	end

	game.setState("menu", data.prev, menu.substate, true)
end

data = {
	cancel = cancel,
	label = "Delete Save Data",
	update = function(self, dt, select)
		if accept then
			if accept ~= select then
				self[accept].text = self[accept].origText
				accept = nil
			end
		end
	end,

	getButton("A", 1),
	getButton("B", 2),
	getButton("C", 3),
	false,
	{
		text = "Reset Unlocks", desc = "Resets unlocks such as characters.",
		action = function(self)
			if accept == 5 then
				love.filesystem.remove("unlocks")
				self.text = "Reset Unlocks!"
				unlocks = {
					denniz = true,
					diana = true
				}
			else
				accept = 5
				self.origText = self.text
				self.text = { colors.red, "Really reset all unlocks?" }
			end
		end
	},
	false,
	{
		text = "Cancel",
		action = cancel
	}
}

return data
