local colors = colors

local levelNames = fileT.require("jnr/level/#names.txt")

local function getButton(slot)
	local file = "save"..tostring(slot)
	if love.filesystem.isFile(file) then
		local data, err = saveLib.read(file)
		if data then
			local time = save.formatTime(data.time or 0)

			local deaths = data.deaths ~= 1 and "s" or ""

			local collect = data.totalCollect or 0
			local totalCollect = jnr.collect.__total or 0/0
			local progress = 100*collect/totalCollect

			return {
				text = {
					colors.white, ("Slot %s: [%s] [%d%%] "):format(slot, time, progress),
					data.hardcore and colors.uhc or colors.gold, data.hardcore and "[UHC]" or "[Normal]"
				},
				desc = {
					colors.white, ("Last seen at [%s].\nCollected [%d/%d (%d%%)] things...\nPlayed a total of [%s] hours.\nDied [%d] time%s in "):format(levelNames[data.hub] or data.hub, collect, totalCollect, progress, time, data.deaths or 0, deaths),
					data.hardcore and colors.uhc or colors.gold, data.hardcore and "Ultra Hardcore Mode" or "Normal Mode",
					colors.white, "."
				},
				action = function()
					save.init(slot)
				end
			}
		else
			print("[Menu]", "Cannot load data in slot "..tostring(slot)..":", err)
			return {
				text = {
					colors.red, ("Slot %s: Corrupted?"):format(slot),
				},
				desc = "Corrupted data cannot be loaded. Delete it via the deletion menu."
			}
		end
	else
		local hardcore = false
		local nosave = slot == nil

		local function swap(self)
			hardcore = not hardcore
			if hardcore then
				self.text[5] = colors.uhc
				self.text[6] = "UHC"

				self.desc[5] = self.text[5]
				self.desc[6] = "Ultra Hardcore Mode"
				self.desc[10] = "Don't even bother trying."
			else
				self.text[5] = colors.gold
				self.text[6] = "Normal"

				self.desc[5] = self.text[5]
				self.desc[6] = "Normal Mode"
				self.desc[10] = "You should be able to beat this..."
			end
		end

		return {
			text = {
				colors.white, (slot and ("Slot %s: New  "):format(slot) or "No-Save Slot "),
				colors.yelora, "[< ",
				colors.gold, "Normal",
				colors.yelora, " >]"
			},
			desc = {
				colors.red, "No data.\n",
				colors.white, "Start a new adventure in ",
				colors.gold, "Normal Mode",
				colors.white, "!\n",
				colors.white, "You should be able to beat this...",
				nosave and colors.red or nil, nosave and "\nDATA WILL NOT BE SAVED ON THIS SLOT!" or nil
			},
			action = function()
				save.init(slot, hardcore)
			end, actionR = swap, actionL = swap
		}
	end
end

music.set "game_at_heart"

return {
	cancel = function()
		game.setState("title", "menu", "main", "main")
	end,
	label = "Main Menu",
	getButton = getButton,

	getButton("A"),
	getButton("B"),
	getButton("C"),
	getButton(nil),
	false,
	{
		text = "Settings", desc = "Change controls, graphics, sounds, etc...",
		action = "options"
	},
	{
		text = "Delete Saved Data", desc = "Clear certain save data slots...\nIf you need to do that.",
		action = "delete"
	},
	false,
	{
		text = "Credits", desc = "You know what they are, don't you?",
		action = "credits"
	},
	{
		text = "Bonus", desc = "Some additional stuff you shouldn't not use...!",
		action = "bonus"
	},
	false,
	{
		text = "Exit", desc = "Goodbye!\n(Quit the game.)",
		action = function() love.event.quit() end
	}
}
