local colors = colors

local data
local function cancel()
	game.queueTask(saveLib.write, "cnf", cnf)
	game.setState("menu", data.prev, menu.substate, true)
end

data = {
	label = "Settings",
	cancel = cancel,
	color = colors.new(200, 100, 0),
	thin = true,

	(function()
		local function action(self)
			window.toggleFullscreen()
			self.text[6] = cnf.fs and "On" or "Off"
		end

		return {
			text = {
				colors.white, "Fullscreen> ",
				colors.yelora, "[< ",
				colors.white, cnf.fs and "On" or "Off",
				colors.yelora, " >]"
			},
			desc = "It's actually using 'Windowed Fullscreen' mode...",
			action = action, actionL = action, actionR = action
		}
	end)(),
	(function()
		local function action(self)
			cnf.vsync = not cnf.vsync
			self.text[6] = (cnf.vsync and "On" or "Off").." [Restart Game!]"
		end

		return {
			text = {
				colors.white, "VSync> ",
				colors.yelora, "[< ",
				colors.white, cnf.vsync and "On" or "Off",
				colors.yelora, " >]"
			},
			desc = "VSync stabilizes rendering but adds a minimal input delay.",
			actionL = action, actionR = action
		}
	end)(),
	{
		text = {
			colors.white, "Shadow Detail> ",
			colors.yelora, "[< ",
			colors.white, cnf.shadow > 0 and ("%d%%"):format(100/cnf.shadow) or "Off",
			colors.yelora, " >]"
		},
		desc = "Sets the detail of real-time-rendered shadows.\nMay affect performance, disable if you experience frame rate issues!",
		actionL = function(self)
			cnf.shadow = cnf.shadow + 1
			if cnf.shadow > 4 then
				cnf.shadow = 0
			end

			local switchToUpdate = false
			if jnr.tiles then
				local canvas = jnr.tiles.canvas
				if canvas.init then
					canvas:update()
				else
					switchToUpdate = true
				end
			end

			self.text[6] = cnf.shadow > 0 and ("%d%%%s"):format(100/cnf.shadow, switchToUpdate and " [Reload!]" or "") or "Off"
		end,
		actionR = function(self)
			cnf.shadow = cnf.shadow - 1
			if cnf.shadow < 0 then
				cnf.shadow = 4
			end

			local switchToUpdate = false
			if jnr.tiles then
				local canvas = jnr.tiles.canvas
				if canvas.init then
					canvas:update()
				else
					switchToUpdate = true
				end
			end

			self.text[6] = cnf.shadow > 0 and ("%d%%%s"):format(100/cnf.shadow, switchToUpdate and " [Reload!]" or "") or "Off"
		end
	},
	{
		text = {
			colors.white, "Maximum Particles> ",
			colors.yelora, "[< ",
			colors.white, tostring(cnf.pmax),
			colors.yelora, " >]"
		},
		desc = "The maximum amount of particles that can be active at any time.\nShould not greatly affect performance.",
		actionL = function(self)
			cnf.pmax = cnf.pmax - 100
			if cnf.pmax < 200 then
				cnf.pmax = 10000
			end

			if jnr.particle then jnr.particle:setMax(cnf.pmax) end

			self.text[6] = tostring(cnf.pmax)
		end,
		actionR = function(self)
			cnf.pmax = cnf.pmax + 100
			if cnf.pmax > 10000 then
				cnf.pmax = 200
			end

			if jnr.particle then jnr.particle:setMax(cnf.pmax) end

			self.text[6] = tostring(cnf.pmax)
		end
	},
	false,
	{
		text = {
			colors.white, "Sound Effects> ",
			colors.yelora, "[< ",
			colors.white, ("%d%%"):format(cnf.sfx),
			colors.yelora, " >]"
		},
		desc = "Sound effect (SFX) volume. (Selecting, hitting stuff, attacking etc...)",
		actionL = function(self)
			cnf.sfx = cnf.sfx - 5
			if cnf.sfx < 0 then
				cnf.sfx = 100
			end

			sounds.setSFXVolume(cnf.sfx*0.01)

			self.text[6] = ("%d%%"):format(cnf.sfx)
		end,
		actionR = function(self)
			cnf.sfx = cnf.sfx + 5
			if cnf.sfx > 100 then
				cnf.sfx = 0
			end

			sounds.setSFXVolume(cnf.sfx*0.01)

			self.text[6] = ("%d%%"):format(cnf.sfx)
		end
	},
	{
		text = {
			colors.white, "Music Volume> ",
			colors.yelora, "[< ",
			colors.white, ("%d%%"):format(cnf.music),
			colors.yelora, " >]"
		},
		desc = "Background music volume.",
		actionL = function(self)
			cnf.music = cnf.music - 5
			if cnf.music < 0 then
				cnf.music = 100
			end

			sounds.setMusicVolume(cnf.music*0.01)
			if music.current then
				if cnf.music == 0 then
					music.current:stop()
				else
					music.current:play()
				end
			end

			self.text[6] = ("%d%%"):format(cnf.music)
		end,
		actionR = function(self)
			cnf.music = cnf.music + 5
			if cnf.music > 100 then
				cnf.music = 0
			end

			sounds.setMusicVolume(cnf.music*0.01)
			if music.current then
				if cnf.music == 0 then
					music.current:stop()
				else
					music.current:play()
				end
			end

			self.text[6] = ("%d%%"):format(cnf.music)
		end
	},
	false,
	{
		text = "Keyboard Setup", action = "options_keyboard"
	},
	{
		text = "Gamepad Setup", action = "options_gamepad"
	},
	{
		text = "Fixed Key Bindings",
		desc = "Open a list of all fixed key bindings.",
		action = "options_fixed"
	},
	false,
	{
		text = "View Screenshots",
		desc = "Open the folder containing all screenshots taken in (hopefully) your file explorer.",
		action = function()
			love.system.openURL("file:///"..love.filesystem.getSaveDirectory().."/screenshots")
		end
	},
	false,
	{
		text = {
			colors.white, "Change Character> ",
			colors.yelora, "[ ",
			colors.white, (fileT.get("jnr/char/#names.txt", cnf.char) or cnf.char),
			colors.yelora, " ]"
		},
		desc = "Select your preferred character.\nThe character will only actually change when the current level changes or is reloaded.",
		action = "char"
	},
	(function()
		local function action(self)
			cnf.multi = not cnf.multi
			input.separate(cnf.multi)

			self.text[6] = (cnf.multi and "On" or "Off").." [Reload!]"
		end

		return {
			text = {
				colors.white, "Multiplayer> ",
				colors.yelora, "[< ",
				colors.white, cnf.multi and "On" or "Off",
				colors.yelora, " >]"
			},
			desc = "Make the game spawn a second player character.\nOne of the characters will be controlled via keyboard, the other via gamepad.\nIf you want to mess around only.",
			action = action, actionL = action, actionR = action
		}
	end)(),
	false,
	{
		text = "Back", action = cancel
	}
}

return data
