local love = love
local input, colors = input, colors

local assigned = {}
for i,v in pairs{"jl", "jr", "ju", "jd", "jj", "ja", "jp", "jc"} do
	assigned[cnf[v]] = true
end

local function setButton(name, bind, cnfId, canDisable)
	return {
		text = {
			colors.white, name.."> ",
			colors.yelora, "[ ",
			colors.white, input.getButtonName(cnf[cnfId], cnf.prompt),
			colors.yelora, " ]"
		},
		action = function(self)
			self.origText = self.text
			self.text = "Hit new button for '"..name.."' or Esc."
			local keypressed = love.keypressed
			local keyreleased = love.keyreleased
			local gamepadpressed = love.gamepadpressed
			local gamepadreleased = love.gamepadreleased
			local gamepadaxis = love.gamepadaxis

			local old

			if canDisable then
				old = "--"
			else
				old = cnf[cnfId]
			end

			input.bindButton(cnf[cnfId], nil)
			assigned[cnf[cnfId]] = nil

			function love.gamepadpressed(joystick, button)
				if not assigned[button] then
					input.bindButton(button, bind)
					cnf[cnfId] = button
					assigned[button] = true

					love.keypressed = keypressed
					love.keyreleased = keyreleased
					love.gamepadpressed = gamepadpressed
					love.gamepadreleased = gamepadreleased
					love.gamepadaxis = gamepadaxis

					self.text = self.origText
					self.text[6] = input.getButtonName(button, cnf.prompt)
				end
			end

			function love.keypressed(key, scancode, isrepeat)
				if key == "escape" then
					input.bindButton(old, bind)
					cnf[cnfId] = old
					assigned[old] = true

					love.keypressed = keypressed
					love.keyreleased = keyreleased
					love.gamepadpressed = gamepadpressed
					love.gamepadreleased = gamepadreleased
					love.gamepadaxis = gamepadaxis

					self.text = self.origText
					self.text[6] = input.getButtonName(old, cnf.prompt)
				end
			end

			local function empty() end
			love.keyreleased = empty
			love.gamepadreleased = empty
			love.gamepadaxis = empty
		end
	}
end

local data
data = {
	label = "Gamepad Setup",
	color = colors.new(200, 100, 0),
	thin = true,

	setButton("Left", "left", "jl"),
	setButton("Right", "right", "jr"),
	setButton("Up", "up", "ju"),
	setButton("Down", "down", "jd"),
	setButton("Up Alt.", "up", "ju2", true),
	setButton("Down Alt.", "down", "jd2", true),
	setButton("Jump/Accept", "jump", "jj"),
	setButton("Attack/Cancel", "attack", "ja"),
	setButton("Pause", "pause", "jp"),
	setButton("Char. Sel.", "cselect", "jc"),
	false,
	{
		text = {
			colors.white, "Axis Threshold> ",
			colors.yelora, "[< ",
			colors.white, ("%d%%"):format(cnf.jt),
			colors.yelora, " >]"
		},
		desc = "How far you have to move sticks on your gamepad until the input is recognized.",
		actionL = function(self)
			cnf.jt = cnf.jt - 5
			if cnf.jt < 5 then
				cnf.jt = 95
			end

			input.bindAxis("leftx", cnf.jt*0.01, "left", "right")
			input.bindAxis("lefty", cnf.jt*0.01, "up", "down")
			input.bindAxis("righty", cnf.jt*0.01, "up", "down")

			self.text[6] = ("%d%%"):format(cnf.jt)
		end,
		actionR = function(self)
			cnf.jt = cnf.jt + 5
			if cnf.jt > 95 then
				cnf.jt = 5
			end

			input.bindAxis("leftx", cnf.jt*0.01, "left", "right")
			input.bindAxis("lefty", cnf.jt*0.01, "up", "down")
			input.bindAxis("righty", cnf.jt*0.01, "up", "down")

			self.text[6] = ("%d%%"):format(cnf.jt)
		end
	},
	false,
	(function()
		local function action(self)
			if cnf.prompt == "xinput" then
				cnf.prompt = "ds4"
			else
				cnf.prompt = "xinput"
			end

			self.text[6] = (cnf.prompt == "xinput" and "XInput/XBox" or "DS4/PlayStation")

			data[1].text[6] = input.getButtonName(cnf.jl, cnf.prompt)
			data[2].text[6] = input.getButtonName(cnf.jr, cnf.prompt)
			data[3].text[6] = input.getButtonName(cnf.ju, cnf.prompt)
			data[4].text[6] = input.getButtonName(cnf.jd, cnf.prompt)
			data[5].text[6] = input.getButtonName(cnf.ju2,cnf.prompt)
			data[6].text[6] = input.getButtonName(cnf.jd2,cnf.prompt)
			data[7].text[6] = input.getButtonName(cnf.jj, cnf.prompt)
			data[8].text[6] = input.getButtonName(cnf.ja, cnf.prompt)
			data[9].text[6] = input.getButtonName(cnf.jp, cnf.prompt)
			data[10].text[6]= input.getButtonName(cnf.jc, cnf.prompt)
		end

		return {
			text = {
				colors.white, "Prompts> ",
				colors.yelora, "[< ",
				colors.white, (cnf.prompt == "xinput" and "XInput/XBox" or "DS4/PlayStation"),
				colors.yelora, " >]"
			},
			desc = "Which kind of gamepad buttons do you want to see?",
			action = action, actionL = action, actionR = action
		}
	end)(),
	false,
	{
		text = "Back", action = "!"
	}
}

return data
