local love = love
local input, colors = input, colors

local log = newLog "[Password]"

local active = false
local keypressed, keyreleased, gamepadpressed, gamepadreleased, gamepadaxis, textinput
local password = {}
local accept, cancel

local data
data = {
	label = "Password",
	color = colors.new(0, 175, 0),

	{
		text = "Enter: ----",
		action = function(self)
			if active then
				log("Resetting callbacks.")

				love.keypressed = keypressed
				love.keyreleased = keyreleased
				love.gamepadpressed = gamepadpressed
				love.gamepadreleased = gamepadreleased
				love.gamepadaxis = gamepadaxis
				love.textinput = textinput

				cnf.kj = accept
				cnf.ka = cancel

				active = false

				self.text = "Enter: "..table.concat(password)..("-"):rep(math.max(4-#password, 0))
				love.keyboard.setTextInput(false)
				love.keyboard.setKeyRepeat(false)
			else
				log("Storing and overriding callbacks...")

				keypressed = love.keypressed
				keyreleased = love.keyreleased
				gamepadpressed = love.gamepadpressed
				gamepadreleased = love.gamepadreleased
				gamepadaxis = love.gamepadaxis
				textinput = love.textinput

				local function empty() end
				love.keyreleased, love.gamepadpressed, love.gamepadreleased, love.gamepadaxis = empty, empty, empty, empty

				function love.keypressed(key, scancode, isrepeat)
					if scancode == "return" and not isrepeat then
						input.hitInput("jump")
					elseif scancode == "backspace" then
						if #password > 0 then
							password[#password] = nil
							self.text = "Type: "..table.concat(password).."_"
						end
					end
				end

				function love.textinput(text)
					if #password < 16 then
						password[#password+1] = text
						if #password < 16 then
							self.text = "Type: "..table.concat(password).."_"
						else
							self.text = "Type: "..table.concat(password)
						end
					end
				end

				accept = cnf.kj
				cancel = cnf.ka
				cnf.kj = "return"
				cnf.ka = "return"

				input.releaseInput("up")
				input.releaseInput("down")

				active = true

				self.text = "Type: "..table.concat(password).."_"
				love.keyboard.setTextInput(true)
				love.keyboard.setKeyRepeat(true)
			end
		end
	},
	{
		text = "Accept",
		action = function(self)
			if #password >= 4 then
				local hash = md5.sumhexa(table.concat(password))
				local rawcode = fileT.get("menu/passwords.txt", hash)

				if rawcode then
					log("Valid password, hash: ", hash, "\nRunning: ", pcall(select(2, pcall(loadstring, rawcode)), data))
				else
					log("Invalid password, hash: ", hash)
					game.message.queue{colors.pRed, "Unknown password. Try again."}
				end
			else
				log("Password too short?!")
				game.message.queue{colors.pRed, "Passwords are at least 4 characters long!"}
			end
		end
	},
	false,
	{
		text = "Cancel",
		action = "!"
	}
}

return data
