local love = love
local max = math.max
local insert, remove = table.insert, table.remove
local pcall, tostring, tonumber = pcall, tostring, tonumber
local assets, colors = assets, colors

if not love.filesystem.isDirectory("screenshots") then
	love.filesystem.createDirectory("screenshots")
end

local items = love.filesystem.getDirectoryItems("screenshots")

for i=#items, 1, -1 do
	local v = items[i]
	if not love.filesystem.isFile("screenshots/"..v) or not v:find("^%d+%-%d+%-%d+_%d+%.%d+%.%d+") then
		remove(items, i)
	end
end

table.sort(items, function(a, b)
	if a:find("%-%d+%.png$") then
		if b:find("%-%d+%.png$") then
			return a < b
		else
			return a < b:gsub("%.png$", "-0.png")
		end
	else
		if b:find("%-%d+%.png$") then
			return a:gsub("%.png$", "-0.png") < b
		else
			return a < b
		end
	end
end)

local data = {
	label = "Screenshots",
	color = colors.new(200, 100, 0),

	{
		text = "View directory...",
		action = function()
			love.system.openURL("file:///"..love.filesystem.getSaveDirectory().."/screenshots")
		end
	},
	false,
	{
		text = "Back", action = "!"
	}
}

for i=max(1, #items-9), #items do
	local path = items[i]

	local Y, M, D, h, m, s = path:match("^(%d+)%-(%d+)%-(%d+)_(%d+)%.(%d+)%.(%d+)")
	local t = tonumber(path:match("%-(%d+)%.png$"))

	insert(data, 1, {
		text = ("'%02d/%02d/%02d at %02d:%02d:%02d %s"):format(Y, M, D, h, m, s, t and "(No. "..tostring(t+1)..")" or ""),
		desc = "Accept to open image.",
		action = function(self)
			love.system.openURL("file:///"..love.filesystem.getSaveDirectory().."/screenshots/"..path)
		end,
	})
end

return data
