local love = love
local allMods = love.filesystem.getDirectoryItems("mods")

if #allMods > 1 then
	local log = newLog "[Mods]"

	mods = {
		__init = true,
		mods = {},
		chars = {},
		levels = {}
	}
	local mods = mods

	local loading

	local function loadMod(path)
		log("Loading mod from '"..path.."'...")
		loading = path

		love.filesystem.mount(path, "", false)

		local thismod = {}
		if love.filesystem.isFile("main.lua") then
			log("\tRunning mod's initialization...")
			thismod = love.filesystem.load("main.lua")() or thismod
		end

		loading = nil
		return thismod
	end

	local editFileT = false
	function mods.char(id, name, unlock)
		if not editFileT then
			log("\tOverriding filetable (fileT) lib to always require...")
			local require = fileT.require
			fileT.load = require
			function fileT.get(file, key)
				return require(file)[key]
			end
		end

		log("\t\tAdded character: id, name, desc:", id, name, desc)
		local new = {
			id = id, name = name, desc = desc, unlock = unlock,
			mod = loading
		}

		fileT.require("jnr/char/#names.txt")[id] = name and tostring(name)
		if unlock then
			fileT.require("jnr/char/#unlck.txt")[id] = tostring(unlock)
		end

		mods.chars[#mods.chars+1] = new
		return new
	end

	function mods.levelInfo(id, name, collect, exclude)
		log("\t\tAdded Level info: id, name, collect:", id, name, collect or "nil")
		local new = {
			id = id, name = name, collect = collect,
			mod = loading
		}

		jnr.collect[id] = tonumber(collect)
		if jnr.collect[id] and not exclude then
			jnr.collect.__total = jnr.collect.__total + jnr.collect[id]
		end
		fileT.require("jnr/level/#names.txt")[id] = tostring(name)

		mods.levels[#mods.levels+1] = new
		return new
	end

	function mods.inject(orig, prev, post)
		if prev and post then
			return function(...)
				prev(...)
				orig(...)
				post(...)
			end
		elseif prev then
			return function(...)
				prev(...)
				orig(...)
			end
		elseif post then
			return function(...)
				orig(...)
				post(...)
			end
		else
			return orig
		end
	end

	for k,v in pairs(allMods) do
		if v ~= "init.lua" then
			local s, e = pcall(loadMod, "mods/"..v)
			if s then
				local vs = v:gsub("%.[^%.]-$", "")
				mods.mods[vs] = e
			else
				log("Error loading:", e)
			end
		end
	end

	return mods
else
	return {
		__init = false
	}
end
