local sfx = sfx

local Actor = {}

function Actor:new(x, y, data)
	self:__new(x, y, data.w or 8, data.h or 17, 0)

	for k,v in pairs(data) do self[k] = v end
	if data.anim then self.anim = data.anim:clone() end
	if data.new then data.new(self) end

	self.xv = 0
	self.yv = 0

	self.toX = self.x
	self.maxXv = 0

	self.canColide = false
	self.__data = data
end

function Actor:duck()
	self.data.duck = true
	local h = self.h * 0.5
	self.y = self.y + h
	self.h = h
end

function Actor:unduck()
	self.data.duck = false
	self.y = self.y - self.h
	self.h = self.h * 2
end

function Actor:fixAnim(id)
	if id == nil then
		self.statusFix = false
	else
		self.statusFix = self:getStatus()
		self.anim:set(id)
	end
end

function Actor:moveActor(toX, xv)
	self.data.moving = true
	self.toX = toX*self.tiles.ts-self.tiles.ts*0.5 - self.w*0.5
	self.maxXv = xv or 60

	if self.toX > self.x then
		self.dir = 1
	elseif self.toX < self.x then
		self.dir = -1
	end
end

function Actor:jumpActor(yv)
	if yv == nil then yv = -210 end
	self.tiles.particle:spawn("dust", -yv*0.02, self.x + self.w * 0.5, self.y + self.h, self.w * 0.3, 1, -self.xv * 0.25)

	self.yv = yv

	sfx.colide:play(self.x + self.w*0.5, self.y + self.h, 0.075)
	self.data.wall = false
	self.data.ground = false
end

function Actor:update(dt)
	self:setAcceleration()

	local moveDir = 0
	if self.data.moving then
		if self.toX > self.x then
			self.xvLimit = self.maxXv
			moveDir = 1
		elseif self.toX < self.x then
			self.xvLimit = -self.maxXv
			moveDir = -1
		end
	end

	self:accelerate(dt)

	self:colide(dt)

	if (moveDir > 0 and self.toX <= self.x) or (moveDir < 0 and self.toX >= self.x) then
		self.xvLimit = 0
		self.data.moving = false
	end

	if self.animate then self:animate(dt) end
	self:spawnParticles(dt)
	self:makeSounds()
end

function Actor:__tostring()
	return ("Actor at %.2f, %.2f"):format(self.x or 0/0, self.y or 0/0)
end

return class("Actor", require("jnr.object"), Actor)
