local utf8 = require "utf8"
local ipairs, type = ipairs, type
local insert = table.insert
local colors = colors

local SlowText = {}

function SlowText:new(text, speed)
	self.timer = 0
	self.progress = 0
	self.speed = speed or 0.05
	self.print = {}
	self.terminated = false

	local color = colors.white
	if type(text) == "string" then
		self.text = {}

		for k, v in ipairs{ utf8.codepoint(text, 1, #text) } do
			insert(self.text, {color, utf8.char(v)})
		end
	else
		self.text = {}

		for i, e in ipairs(text) do
			if type(e) == "table" then
				color = e
			else
				for k, v in ipairs{utf8.codepoint(e, 1, #e)} do
					insert(self.text, {color, utf8.char(v)})
				end
			end
		end
	end

	self:update(self.speed)
	self.length = #self.text
end

function SlowText:update(dt)
	if self.terminated then
		self.terminated = self.terminated + dt
	else
		self.timer = self.timer + dt

		while self.timer >= self.speed do
			self.timer = self.timer - self.speed
			self.progress = self.progress + 1

			local char = self.text[self.progress]
			if char ~= nil then
				if self.lastColor ~= char[1] then
					self.lastColor = char[1]
					self.print[#self.print+1] = self.lastColor
					self.print[#self.print+1] = ""
				end

				if #self.print == 0 then return end

				if char[2] == " " then
					if self.lastChar ~= " " then
						self.print[#self.print] = self.print[#self.print] .. char[2]
						self.timer = self.timer + self.speed
					end
				else
					self.print[#self.print] = self.print[#self.print] .. char[2]
				end

				if char[2] == "\n" then self.timer = self.timer - (self.speed * 5) end

				self.lastChar = char[2]
			else
				self.terminated = 0
				break
			end
		end
	end
end

function SlowText:__tostring()
	return "SlowText"
end

return class("SlowText", SlowText)
