local love = love
local sin = math.sin
local window, colors = window, colors

local log = newLog "[Title]"

local title = {}

local orig, cont, next, time, bg, graphic

function title.init(...)
	next = {...}

	orig = {
		love.keypressed, love.gamepadpressed
	}

	log("Setting temporary callbacks...")
	function love.keypressed(key, scancode, isrepeat)
		cont = true
		orig[1](key, scancode, isrepeat)
	end
	function love.gamepadpressed(joystick, button)
		cont = true
		orig[2](joystick, button)
	end

	time = 0

	bg = {}
	local Tiles = require "jnr.tiles"
	bg.setBackground, bg.updateBackground, bg.drawBackground = Tiles.setBackground, Tiles.updateBackground, Tiles.drawBackground
	bg:setBackground {
		{"forest/skybox", 0, "x"},
		{"forest/cloud1", 0.3, "x", 6},
		{"forest/cloud2", 0.4, "x", 9},
		{"forest/hills" , 0.5, "x"},
		{"forest/cloud3", 0.6, "x", 18},
	} -- BG data from jnr/tiles/forest.lua

	graphic = {}
	graphic.image = assets.image("assets/menu.png")
	graphic.logo  = love.graphics.newQuad(0, 27, 256, 64, 256, 256)

	music.set "game_at_heart"
end

function title.next(...)
	next = {...}
end

function title.deinit()
	log("Restoring callbacks...")
	love.keypressed, love.gamepadpressed = unpack(orig)

	log("Clearing data...")
	orig, cont, next, time, bg, graphic = nil
end

function title.update(dt)
	if cont then
		game.setState(unpack(next))
	end
end

function title.draw(dt)
	bg:updateBackground(dt)

	time = time + dt

	love.graphics.push()
	love.graphics.scale(window.s)

	bg:drawBackground(time * 40, 0, window.vw, window.vh)

	love.graphics.draw(graphic.image, graphic.logo, window.vw*0.5, window.vh*0.4 + sin(time*3)*10, sin(time)*0.075, 2, 2, 128, 32)

	local y = window.vh*0.7 + sin(time*3)*6
	local r = sin(time*0.7)*0.05
	love.graphics.setColor(colors.black)
	love.graphics.printf("Hit any key/button!", window.vw*0.5+2, y+2, window.vw*0.5, "center", r, 2, 2, window.vw*0.25, 5)
	love.graphics.setColor(colors.white(sin(time*6)*0.5+0.5, sin(time*6)*0.25+0.75, 1, 1))
	love.graphics.printf("Hit any key/button!", window.vw*0.5, y, window.vw*0.5, "center", r, 2, 2, window.vw*0.25, 5)

	love.graphics.setColor(colors.white)

	love.graphics.pop()
end

return title
